import { MaterializeModule } from 'angular2-materialize';
import { BrowserModule } from '@angular/platform-browser';
import { NgModule } from '@angular/core';
import {RouterModule, Routes} from '@angular/router';
import { FormsModule } from '@angular/forms';
import { HttpModule } from '@angular/http';
import { FlashMessagesModule } from 'angular2-flash-messages';
import { DragulaModule } from 'ng2-dragula/ng2-dragula';
import {MatDialogModule, MatButtonModule } from '@angular/material';
import { NoopAnimationsModule } from '@angular/platform-browser/animations';


import { AppComponent } from './app.component';
import { LoginComponent } from './login/login.component';
import { RegisterComponent } from './register/register.component';
import { HomeComponent } from './home/home.component';
import { ProfileComponent } from './profile/profile.component';
import { AuthGuard } from './guards/auth.guard';
import { HeaderComponent } from './header/header.component';
import { LaboratoryComponent } from './laboratory/laboratory.component';
import { SpellbookComponent } from './spellbook/spellbook.component';
import { ConfirmDialogComponent } from './confirm-dialog/confirm-dialog.component';
import { RenameDialogComponent } from './rename-dialog/rename-dialog.component';
import { AudioPlayerComponent } from './audio-player/audio-player.component';
import { PagelistDialogComponent } from './pagelist-dialog/pagelist-dialog.component';
import { InfoDialogComponent } from './info-dialog/info-dialog.component';
import { ErrorComponent } from './error/error.component';

import { ValidateService } from './services/validate.service';
import { AuthService } from './services/auth.service';
import { MathService } from './services/math.service';
import { SvgService } from './services/svg.service';
import { DialogService } from './services/dialog.service';
import { AudioPlayerService } from './services/audio-player.service';
import { ErrorService } from './services/error.service';
import { SliderDialogComponent } from './slider-dialog/slider-dialog.component';



const appRoutes:Routes = [
    {path:'', component: HomeComponent},
    {path:'register', component: RegisterComponent},
    {path:'login', component: LoginComponent},
    {path:'error', component: ErrorComponent},
    {path:'profile', component: ProfileComponent, canActivate:[AuthGuard]},
    {path:'laboratory', component: LaboratoryComponent, canActivate:[AuthGuard]},
    {path:'spellbook', component: SpellbookComponent, canActivate:[AuthGuard]}
]

@NgModule({
  declarations: [
    AppComponent,
    LoginComponent,
    RegisterComponent,
    HomeComponent,
    ProfileComponent,
    HeaderComponent,
    LaboratoryComponent,
    SpellbookComponent,
    ConfirmDialogComponent,
    RenameDialogComponent,
    AudioPlayerComponent,
    PagelistDialogComponent,
    InfoDialogComponent,
    ErrorComponent,
    SliderDialogComponent
  ],
  imports: [
    MaterializeModule,
    BrowserModule,
    FormsModule,
    HttpModule,
    MatDialogModule,
    MatButtonModule,
    DragulaModule,
    NoopAnimationsModule,
    FlashMessagesModule.forRoot(),
    RouterModule.forRoot(appRoutes)
  ],
  providers: [
    ValidateService,
    AuthService,
    AuthGuard,
    MathService,
    SvgService,
    DialogService,
    AudioPlayerService,
    ErrorService
  ],
  entryComponents: [ConfirmDialogComponent, RenameDialogComponent, PagelistDialogComponent, InfoDialogComponent, SliderDialogComponent],
  bootstrap: [AppComponent]
})
export class AppModule { }
