import { Component, OnInit, ViewChild, ElementRef } from '@angular/core';
import { AudioPlayerService } from '../services/audio-player.service';
import { AuthService } from '../services/auth.service';

@Component({
  selector: 'app-audio-player',
  templateUrl: './audio-player.component.html',
  styleUrls: ['./audio-player.component.scss']
})
export class AudioPlayerComponent implements OnInit {

  constructor(private audioService:AudioPlayerService,
              private authService:AuthService) { }

  pauseSwitch:boolean;
  source:string;
  soundIcon:string;
  arcaneId:number;
  timer = {
    current: "00:00",
    duration: "00:00"
  }
  newSource:boolean; //prevent canplaythrough to trigger play loop

  @ViewChild('audio') audio:ElementRef;
  @ViewChild('progress') progress:ElementRef;
  @ViewChild('audioCard') audioCard:ElementRef;

  ngOnInit() {
    this.pauseSwitch = false;
    this.source = "";
    this.soundIcon = "";
    this.newSource = false;
    this.arcaneId = -1;

    this.audioService.currentAudioSrc.subscribe((arcane) => {
      if(arcane && arcane.id == this.arcaneId) {
        this.play();
      }
      else if(arcane && this.audio) {
        this.source = this.authService.webUrl + "sounds/" + arcane.fileName + ".WAV";
        this.soundIcon = this.authService.webUrl + "arcanes/" + arcane.fileName + ".svg";
        this.audio.nativeElement.load();
        this.newSource = true;
        this.arcaneId = arcane.id;
      }
      else {
        this.pauseSwitch = false;
        this.source = "";
        this.soundIcon = "";
        this.newSource = false;
        this.arcaneId = -1;
      }
    });

    this.audioService.currentSoundOnly.subscribe((f) => {
      if(f) {
        this.source = f;
        this.soundIcon = "";
        this.audio.nativeElement.load();
        this.newSource = true;
        this.arcaneId = -1;
      }
      else {
        this.pauseSwitch = false;
        this.source = "";
        this.soundIcon = "";
        this.newSource = false;
        this.arcaneId = -1;
      }
    });
  }

  play() {
    if(!this.source) {
      return;
    }
    this.audio.nativeElement.play();
    this.pauseSwitch = true;
    this.timer.duration = this.convertTime(this.audio.nativeElement.duration);
    this.timer.current = this.convertTime(0);
  }

  pause() {
    this.audio.nativeElement.pause();
    this.pauseSwitch = false;
  }

  stop() {
    if(!this.source) {
      return false;
    }
    this.pause();
    this.audio.nativeElement.currentTime = 0.0;
  }

  updateProgress() {
    const percent = Math.floor(this.audio.nativeElement.currentTime / this.audio.nativeElement.duration*100);
    this.progress.nativeElement.style.width = String(percent) + "%";
    this.timer.current = this.convertTime(this.audio.nativeElement.currentTime);
  }

  loaded() {
    if(this.newSource) {
      this.play();
      this.newSource = false;
    }
  }

  convertTime(time) {
    let min = Math.floor(time/60);
    let res = "";
    if(min < 10) {
      res = "0" + String(min) + ":";
    }
    else {
      res = String(min) + ":";
    }

    let sec = Math.floor(time%60);
    if(sec < 10) {
      res += "0" + String(sec);
    }
    else {
      res += String(sec);
    }
    return res;
  }

  volumeChange(event) {
    this.audio.nativeElement.volume = event.target.value/100;
  }

  hoverIn(event) {
    let styles = getComputedStyle(this.audioCard.nativeElement);
    if(this.source && styles.display == "none") {
      this.audioCard.nativeElement.style.display = "block";
      let width = 200;
      this.audioCard.nativeElement.style.left = String(event.clientX - width) + "px";
    }
  }

  hoverOut() {
    this.audioCard.nativeElement.style.display = "none";
  }
}
