import { Component, OnInit, Renderer2, ViewChild, ElementRef } from '@angular/core';
import { AuthService } from '../services/auth.service';
import { ErrorService } from '../services/error.service';
import { MathService } from '../services/math.service';
import { SvgService } from '../services/svg.service';
import { ValidateService } from '../services/validate.service';
import { DialogService } from '../services/dialog.service';
import { DragulaService } from 'ng2-dragula/ng2-dragula';
import { AudioPlayerService } from '../services/audio-player.service';
import { DomSanitizer } from '@angular/platform-browser'

@Component({
  selector: 'app-laboratory',
  templateUrl: './laboratory.component.html',
  styleUrls: ['./laboratory.component.scss'],
  viewProviders: [DragulaService]
})
export class LaboratoryComponent implements OnInit {

  works:any[];
  newWork:string;
  currentWorkspace:any;
  aura:number;

  generateButtonText:string;
  @ViewChild('generateButton') generateButton:ElementRef;
  @ViewChild('generateButtonMobile') generateButtonMobile:ElementRef;

  listenButtonText:string;
  @ViewChild('listenButton') listenButton:ElementRef;
  @ViewChild('listenButtonMobile') listenButtonMobile:ElementRef;
  workspaceChanged:boolean;

  dragElement:any;

  formulaLine:any[];
  formulaHierarchy:any[];
  showWorkspace:boolean;

  elementList:any[];

  subElement = {
    color: [{type: "col", value: null}],
    fonc1: [{type: "F1", value: null}],
    fonc2: [{type: "F2", value: null}],
    mod: [{type: "M", value: null}],
    param: [{type: "K/env", value: null}, {type: "mod", value: null}],
  }

  enableElementReturn:boolean;

  elementLine:any[];

  constructor(private authService:AuthService,
              private ms:MathService,
              private renderer:Renderer2,
              private errorService:ErrorService,
              private svgService:SvgService,
              private validateService:ValidateService,
              private dialogService:DialogService,
              private audioPlayerService:AudioPlayerService,
              private dragula:DragulaService,
              private sanitized:DomSanitizer) { }

  ngOnInit() {
    this.showWorkspace = false;
    this.works = new Array();

    this.dragElement = null;
    this.formulaLine = null;
    this.elementList = null;
    this.formulaHierarchy = null;
    this.elementLine = null;
    this.enableElementReturn = false;
    this.generateButtonText = "Générer";
    this.listenButtonText = "Écouter";
    this.workspaceChanged = true;

    this.currentWorkspace = {
      name: "",
      id: -1,
      content: {size: 0}
    };

    this.authService.authGet("laboratory").subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        if(!data.empty) {
          this.works = data.list;
          this.currentWorkspace = data.current;
          this.showWorkspace = true;
        }
        this.elementList = data.elementList;
        this.elementLine = this.elementList;
        this.aura = data.aura;
      });
    });

    //register changes on workspace list after drop
    this.dragula.drop.subscribe((value) => {
      let [bag, el, target, source, sibling] = value.splice(0);

      if(!el) {
        return;
      }

      for(let i=0; i < this.works.length; i++) {
        this.renderer.setAttribute(target.children[i], 'data-index', String(i));
      }

      let shift = 0;
      if(bag == "bag-work") {
        let dst = this.works.length-1;
        if(sibling) {
          if(el.getAttribute('data-index') < sibling.getAttribute('data-index')) {
            shift = -1;
          }
          dst = Number(sibling.getAttribute('data-index')) + shift;
        }
        this.ms.arrayMoveElement(this.works, Number(el.getAttribute('data-index')), dst);
        for(let i=0; i < this.works.length; i++) {
          this.renderer.setAttribute(target.children[i], 'data-index', String(i));
        }
      }

      this.authService.authPost("laboratory/moveWorkspace", "application/json", this.works).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {});
      });
    });
  }

  //############## MANAGE WORKSPACES ####################################

  addNewWork() {
    if(this.newWork && this.validateService.validateListElem(this.newWork) &&
      !this.validateService.isIn(this.works, this.newWork)) {

      this.authService.authPost("laboratory/newWorkspace", "text/plain", this.newWork).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {
          let work = {
            name: data.workspace.name,
            id: data.workspace.id
          };
          this.works.push(work);
          this.currentWorkspace = data.workspace;
          this.newWork = undefined;
          this.showWorkspace = true;
          this.formulaLine = null;
          this.formulaHierarchy = null;
          this.elementList = data.elements;
          this.elementLine = this.elementList;
        });
      });
    }
    else {
      this.newWork = "";
    }
  }

  changeWorkspace(w) {
    if(w.id != this.currentWorkspace.id) {
      this.authService.authPost("laboratory/changeWorkspace", "application/json", w).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {
          this.currentWorkspace = data.workspace;
          this.formulaLine = null;
          this.formulaHierarchy = null;
          this.workspaceChanged = true;
          this.elementList = data.elements;
          this.elementLine = this.elementList;
        });
      });
    }
  }

  deleteWorkspace() {
    this.dialogService.confirm("Confirmation", "Voulez-vous vraiment supprimer cet espace de travail?").subscribe(res => {
      if(res) {
        for(let i=0; i < this.works.length; i++) {
          if(this.works[i].name == this.currentWorkspace.name) {
            const id = this.works[i].id;
            this.works.splice(i, 1);

            this.authService.authPost("laboratory/deleteWorkspace", "application/json", {list: this.works, id: id}).subscribe((data, err) => {
              this.errorService.serverError(err, data, () => {
                if(data.empty) {
                  this.showWorkspace = false;
                  this.currentWorkspace = {name: "", id: -1, content: undefined};
                }
                else {
                  this.currentWorkspace = data.workspace;
                  this.elementList = data.elements;
                }
                this.formulaLine = null;
                this.formulaHierarchy = null;
                this.elementLine = this.elementList;
                this.enableElementReturn = false;
              });
            });
            break;
          }
        }
      }
    });
  }

  renameWorkspace() {
    this.dialogService.rename("Renommer", "Nouveau nom :").subscribe(res => {
      if(res && this.validateService.validateListElem(res) && !this.validateService.isIn(this.works, res)) {
        for(let w of this.works) {
          if(w.name == this.currentWorkspace.name) {
            this.currentWorkspace.name = res;
            w.name = res;

            this.authService.authPost("laboratory/renameWorkspace", "application/json", {list: this.works, id: w.id, name: res}).subscribe((data, err) => {
              this.errorService.serverError(err, data, () => {});
            });
            break;
          }
        }
      }
    });
  }

  duplicateWorkspace(event) {
    let id = event.target.parentElement.getAttribute('data-index');
    this.dialogService.rename("Copie de l'espace de travail", "Nom de la copie :").subscribe(res => {
      if(res && this.validateService.validateListElem(res) && !this.validateService.isIn(this.works, res)) {
        this.authService.authPost("laboratory/duplicateWorkspace", "application/json", {newName: res, id: id}).subscribe((data, err) => {
          this.errorService.serverError(err, data, () => {
            this.works.push(data);
          });
        });
      }
    });
  }

  //####################### GENERATE ###################################

  generate() {
    if(!this.validateService.checkElementQuantity(this.elementList)) {
      this.dialogService.info("Création impossible", "Vous ne possédez pas tous les éléments utilisés dans la formule.");
      return;
    }
    if(this.validateService.checkFormula(this.currentWorkspace.content)) {
      this.dialogService.confirm("Conditions d'utilisation", "J'accepte <a href=\"#\" target=\"blank\">les conditions d'utilisation</a> concernant le son qui sera créé.", true).subscribe(res => {
        if(res) {
          this.renderer.addClass(this.generateButton.nativeElement, "disabled");
          this.renderer.addClass(this.generateButtonMobile.nativeElement, "disabled");
          this.generateButtonText = "En cours...";
          this.authService.authPost("generate", "application/json", this.currentWorkspace).subscribe((data, err) => {
            this.errorService.serverError(err, data, () => {
              this.generateButtonText = "Générer";
              if(!data.success) {
                this.dialogService.info("Création impossible", data.msg);
              }
              else {
                this.aura = data.aura;
                this.elementList = data.elements;
                this.elementLine = this.elementList;
              }
            });
          });
        }
      });
    }
    else {
      this.dialogService.info("Formule incorrecte", "Tous les éléments de la formule ne sont pas correctement remplis.");
    }
  }

  play() {
    if(!this.workspaceChanged) {
      this.audioPlayerService.changeAudioOnly(this.authService.webUrl + "sounds/tmp/" + this.currentWorkspace.author + ".WAV");
    }
    else if(this.validateService.checkFormula(this.currentWorkspace.content)) {
      this.renderer.addClass(this.listenButton.nativeElement, "disabled");
      this.renderer.addClass(this.listenButtonMobile.nativeElement, "disabled");
      this.listenButtonText = "Patientez...";
      this.authService.authPost("generate/listen", "application/json", this.currentWorkspace).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {
          this.renderer.removeClass(this.listenButton.nativeElement, "disabled");
          this.renderer.removeClass(this.listenButtonMobile.nativeElement, "disabled");
          this.listenButtonText = "Écouter";
          if(!data.success) {
            this.dialogService.info("Création impossible", data.msg);
          }
          else {
            this.renderer.removeClass(this.generateButton.nativeElement, "disabled");
            this.renderer.removeClass(this.generateButtonMobile.nativeElement, "disabled");
            this.workspaceChanged = false;
            this.audioPlayerService.changeAudioOnly(this.authService.webUrl + "sounds/tmp/" + this.currentWorkspace.author + ".WAV#" + new Date().getTime());
          }
        });
      });
    }
    else {
      this.dialogService.info("Formule incorrecte", "Tous les éléments de la formule ne sont pas correctement remplis.");
    }
  }


  //################# WORKSPACE #############################


  displayList(el) {
    this.formulaHierarchy = new Array(el);
    this.formulaLine = this.currentWorkspace.content[el];
  }

  displayEleValues(name) {
    for(let e of this.elementList) {
      if(e.name == name && e.content != null) {
        this.enableElementReturn = true;
        this.elementLine = e.content;
        break;
      }
    }
  }

  navigateTree(el, id) {
    if(Array.isArray(el.value)) {
      this.formulaHierarchy.push(id);
      this.formulaLine = el.value;
    }
  }

  backFormula() {
    if(this.formulaHierarchy && this.formulaHierarchy.length > 1) {
      this.formulaHierarchy.pop();
      this.formulaLine = this.currentWorkspace.content[this.formulaHierarchy[0]];
      for(let i=1; i < this.formulaHierarchy.length; i++) {
        this.formulaLine = this.formulaLine[this.formulaHierarchy[i]].value;
      }
    }
  }

  displayValue(el) {
    if(el.value != null) {
      switch(el.type) {
        case "C":
        case "K":
        case "M":
        case "F1":
        case "F2":
        case "T":
          return el.value;
        case "K/env":
          if(Array.isArray(el.value)) {
            return "env";
          }
        case "mod":
        case "fonc1":
        case "fonc2":
        case "color":
          return el.value[0].value;
        case "param":
          return el.name;
        default:
          return el.type;
      }
    }
    else {
      switch(el.type) {
        case "K/nil":
          return "K_nil";
        case "K/env":
          return "K_env";
        default:
          return el.type;
      }
    }
  }

  testIconDisplay(el, isSvg) {
    if(isSvg) {
      if((el.type == 'K' || el.type == 'K/nil' || el.type == 'K/env') && (el.value != null && !Array.isArray(el.value))) {
        return true;
      }
      return false;
    }
    else {
      if(el.value == null || (el.type == 'K/env' && Array.isArray(el.value)) || (el.type != 'K' && el.type != 'T' && el.type != 'K/nil' && el.type != 'K/env')) {
        return true;
      }
      return false;
    }
  }

  displayDelete(el) {
    return el.value != null && el.type != 'M' &&
           el.type != 'F1' && el.type != 'F2' &&
           el.type != 'param' && el.type != 'env' &&
           el.type != 'col';
  }

  deleteElem(el, index) {
    switch(el.type) {
      case "C":
      case "K/env":
      case "K":
      case "K/nil":
        this.formulaLine[index].value = null;
        break;
      case "T":
        if(this.formulaLine.length > 2) {
          this.modifyEnv(false, index);
        }
      case "mod":
      case "fonc1":
      case "fonc2":
      case "color":
        this.formulaLine.splice(index, 1);
        break;
      default:
        break;
    }

    this.authService.authPost("laboratory/saveWorkspace", "application/json", this.currentWorkspace).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        this.elementList = data.elem;
        let found = false;
        for(let e of this.elementList) {
          if(e.content != null && e.content[0].type == this.elementLine[0].type) {
            this.elementLine = e.content;
            found = true;
          }
        }
        if(!found) {
          this.elementLine = this.elementList;
        }
        this.workspaceChanged = true;
        this.renderer.addClass(this.generateButton.nativeElement, "disabled");
        this.renderer.addClass(this.generateButtonMobile.nativeElement, "disabled");
      });
    });
  }

  //###################### DRAG AND DROP ################################

  dragging(event, e) {
    let ref = JSON.parse(JSON.stringify(e));
    if(ref.content === null) {
      if(ref.type == "K") {
        ref.name = -1;
      }
      event.dataTransfer.setData('text/plain', JSON.stringify(ref));
      this.dragElement = ref;
    }
  }

  dropping(event) {
    event.preventDefault();
    const data = JSON.parse(event.dataTransfer.getData('text/plain'));
    const index = Number(event.target.getAttribute('data-index'));
    let needSave = false;

    //if drop on the correct box
    if(this.formulaLine[index].type == data.type ||
       (this.formulaLine[index].type == "K/nil" && data.type == "K") ||
       (this.formulaLine[index].type == "K/env" && (data.type == "K" || data.type == "env"))) {
      switch(data.type) {
        case "C":
        case "K":
          this.formulaLine[index].value = data.name;
          break;
        case "env":
          this.formulaLine[index].value = new Array({type: "K", value: null});
          for(let i=0; i < this.currentWorkspace.content.time.length -2; i++) {
            this.formulaLine[index].value.push({type: "K/nil", value: null});
          }
          this.formulaLine[index].value.push({type: "K", value: null});
          break;
        case "fonc1":
          // do not add the same function
          for(let e of this.formulaLine) {
            if(e.value != null) {
              //only one surd function
              if(data.name == e.value[0].value || (e.value[0].value.includes("surd") && data.name.includes("surd"))) {
                return;
              }
            }
          }
        case "mod":
        case "fonc2":
        case "color":
          this.formulaLine[index].value = JSON.parse(JSON.stringify(this.subElement[data.type]));
          this.formulaLine[index].value[0].value = data.name;


          //push the number of param for this function
          for(let i=0; i < data.param.length; i++) {
            this.formulaLine[index].value.push({type: "param", name: data.param[i], value: JSON.parse(JSON.stringify(this.subElement.param))});
          }
          if(index == this.formulaLine.length-1) {
            this.formulaLine.push({type: data.type, value: null});
          }
          break;
        case "T":
          if(this.formulaLine[index].name != data.name) {
            //rescale 1-5 to 0-20
            let timeValue = (Number(data.name.substring(1, 2))-1)*4;
            this.formulaLine[index].value = this.ms.timeScale(timeValue);
            this.formulaLine[index].name = data.name;

            if(index == this.formulaLine.length-1) {
              this.formulaLine.push({type: "T", value: null});
              if(this.formulaLine.length >= 3) {
                this.modifyEnv();
              }
            }
          }
          break;
        default:
          break;
      }
      needSave = true;
    }
    this.dragElement = null;
    if(needSave) {
      this.authService.authPost("laboratory/saveWorkspace", "application/json", this.currentWorkspace).subscribe((data, err) => {
        this.errorService.serverError(err, data, () => {
          this.elementList = data.elem;
          let found = false;
          for(let e of this.elementList) {
            if(e.content != null && e.content[0].type == this.elementLine[0].type) {
              this.elementLine = e.content;
              found = true;
            }
          }
          if(!found) {
            this.elementLine = this.elementList;
          }
          this.workspaceChanged = true;
          this.renderer.addClass(this.generateButton.nativeElement, "disabled");
          this.renderer.addClass(this.generateButtonMobile.nativeElement, "disabled");
        });
      });
    }
  }

  modifyEnv(add = true, index = 0) {
    let ref = this.currentWorkspace.content;
    for(let r in ref) {
      if(r != "size" && r != "time") {
        this.recModifyEnv(ref[r], add, index);
      }
    }
  }

  recModifyEnv(arr, add, index) {
    if(arr == null) {
      return;
    }
    for(let a of arr) {
      switch(a.type) {
        case "K/env":
         if(!Array.isArray(a.value)) {
           break;
         }
        case "env":
          if(add) {
            let pos = this.currentWorkspace.content.time.length -2;
            a.value.splice(pos, 0, {type: "K/nil", value: null});
          }
          else {
            let del = a.value.splice(index+1, 1);
            //first and last must be of type K
            if(a.value[a.value.length-1].type == "K/nil") {
              a.value[a.value.length-1].type = "K";
            }
          }
          break;
        case "param":
        case "mod":
        case "func1":
        case "func2":
        case "color":
          this.recModifyEnv(a.value, add, index);
          break;
        default:
          break;
      }
    }
  }

  changeElementValue(e) {
    if(e.type == "T") {
      let timeValue = Number((e.name.substring(1, 2) - 1));
      this.dialogService.sliderValue("Valeur de temps", this.ms.timeScale(timeValue*4), this.ms.timeScale((timeValue-1)*4), e).subscribe(res => {
        if(res != null) {
          e.value = res;
          this.authService.authPost("laboratory/saveWorkspace", "application/json", this.currentWorkspace).subscribe((data, err) => {
            this.errorService.serverError(err, data, () => {
              this.elementList = data.elem;
              this.workspaceChanged = true;
              this.renderer.addClass(this.generateButton.nativeElement, "disabled");
              this.renderer.addClass(this.generateButtonMobile.nativeElement, "disabled");
            });
          });
        }
      });
    }
    else if(!Array.isArray(e.value)) {
      this.dialogService.sliderValue("Constante", -1, 1, e).subscribe(res => {
        if(res != null) {
          e.value = res;
          this.authService.authPost("laboratory/saveWorkspace", "application/json", this.currentWorkspace).subscribe((data, err) => {
            this.errorService.serverError(err, data, () => {
              this.elementList = data.elem;
              this.workspaceChanged = true;
              this.renderer.addClass(this.generateButton.nativeElement, "disabled");
              this.renderer.addClass(this.generateButtonMobile.nativeElement, "disabled");
            });
          });
        }
      });
    }
  }

}
