import { Component, OnInit } from '@angular/core';
import { AuthService } from '../services/auth.service';
import { Router } from '@angular/router';
import { FlashMessagesService } from 'angular2-flash-messages';
import { ValidateService } from '../services/validate.service';
import { ErrorService } from '../services/error.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss']
})
export class LoginComponent implements OnInit {
  username:String;
  password:String;

  constructor(private authService:AuthService,
              private flashMessage:FlashMessagesService,
              private validateService:ValidateService,
              private errorService:ErrorService,
              private router:Router) { }

  ngOnInit() {
  }

  onLoginSubmit() {
    const user = {
      username: this.username,
      password: this.password
    }

    if(!this.validateService.validateLogin(user)){
      this.flashMessage.show("Tous les champs ne sont pas complétés", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    this.authService.authenticateUser(user).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        if(data.success){
          this.authService.storeUserData(data.token, data.user);
          this.router.navigate(['/']);
        }
        else {
          this.password = undefined;
          this.flashMessage.show(data.msg, {cssClass: this.validateService.flashMsgClass, timeout: 5000});
          this.router.navigate(['/login']);
        }
      });
    });
  }

}
