import { Component, OnInit } from '@angular/core';
import { AuthService } from '../services/auth.service';
import { ValidateService } from '../services/validate.service';
import { FlashMessagesService } from 'angular2-flash-messages';
import { Router } from '@angular/router';
import { ErrorService } from '../services/error.service';

@Component({
  selector: 'app-profile',
  templateUrl: './profile.component.html',
  styleUrls: ['./profile.component.scss']
})
export class ProfileComponent implements OnInit {
  user:any;
  password:String;
  changeEmail:boolean;
  changePassword:boolean;
  newEmail:String;
  newPassword:String;
  confirmNewPassword:String;

  constructor(private authService:AuthService,
    private validateService:ValidateService,
    private flashMessage:FlashMessagesService,
    private errorService:ErrorService,
    private router:Router) { }

  ngOnInit() {
    this.changeEmail = false;
    this.changePassword = false;

    this.authService.authGet("users/profile").subscribe((profile, err) => {
      this.errorService.serverError(err, profile, () => {
        this.user = profile.user;
      });
    });
  }

  clearFields() {
    this.password = undefined;
    this.newEmail = undefined;
    this.newPassword = undefined;
    this.confirmNewPassword = undefined;
  }

  onEmailSubmit() {
    if(this.password == undefined || this.newEmail == undefined) {
      this.flashMessage.show("Tous les champs ne sont pas complétés", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    if(!this.validateService.validateEmail(this.newEmail)) {
      this.flashMessage.show("Format d'adresse e-mail non valide", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    const info = {
      type: 'email',
      email: this.newEmail,
      password: this.password
    };

    //push new email
    this.authService.authPost("users/updateUserInfo", "application/json", info).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        if(!data.success) {
          this.password = undefined;
          this.flashMessage.show(data.msg, {cssClass: this.validateService.flashMsgClass, timeout: 5000});
        }
        else {
          this.user.email = this.newEmail;
          this.clearFields();
          this.changeEmail = false;
        }
      });
    });
  }

  onPasswordSubmit() {
    if(!this.validateService.validatePassword(this.newPassword)){
      this.clearFields();
      this.flashMessage.show("Le mot de passe doit être compris entre 8 et 20 caractères\n et avoir au moins un chiffre une majuscule et une minuscule", {cssClass: this.validateService.flashMsgClass, timeout: 8000});
      return false;
    }

    if(this.newPassword != this.confirmNewPassword) {
      this.clearFields();
      this.flashMessage.show("Les mots de passe ne correspondent pas", {cssClass: this.validateService.flashMsgClass, timeout:5000});
      return false;
    }

    const info = {
      type: 'password',
      newPassword: this.newPassword,
      password: this.password
    };

    //push new password
    this.authService.authPost("users/updateUserInfo", "application/json", info).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        if(!data.success) {
          this.clearFields();
          this.flashMessage.show(data.msg, {cssClass: this.validateService.flashMsgClass, timeout: 5000});
        }
        else {
          this.authService.logout();
          this.router.navigate(['/login']);
        }
      });
    });
  }
}
