import { Component, OnInit } from '@angular/core';
import { ValidateService } from '../services/validate.service';
import { AuthService } from '../services/auth.service';
import { Router } from '@angular/router';
import { FlashMessagesService } from 'angular2-flash-messages';
import { ErrorService } from '../services/error.service';

@Component({
  selector: 'app-register',
  templateUrl: './register.component.html',
  styleUrls: ['./register.component.scss']
})
export class RegisterComponent implements OnInit {
  username:String;
  email:String;
  password:String;
  confirmPwd:String;
  license: false;

  constructor(private validateService:ValidateService,
              private flashMessage:FlashMessagesService,
              private authService:AuthService,
              private errorService:ErrorService,
              private router:Router) { }

  ngOnInit() {
  }

  onRegisterSubmit(){
    const user = {
      username: this.username,
      email: this.email,
      password: this.password
    }

    if(!this.license) {
      this.flashMessage.show("Vous devez accepter les termes de la licence pour vous inscrire", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    if(!this.validateService.validateRegister(user)) {
      this.flashMessage.show("Tous les champs ne sont pas complétés", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    if(!this.validateService.validateEmail(user.email)) {
      this.flashMessage.show("Format d'adresse e-mail non valide", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    if(!this.validateService.availableUsername(user.username)) {
      this.flashMessage.show("Ce nom d'utilisateur n'est pas disponible", {cssClass: this.validateService.flashMsgClass, timeout: 5000});
      return false;
    }

    if(!this.validateService.validateUsername(user.username)) {
      this.flashMessage.show("Le nom d'utilisateur doit avoir au minimum 3 caractères et peut contenir des lettres, des chiffres et les caractères - et _", {cssClass: this.validateService.flashMsgClass, timeout: 8000});
      return false;
    }

    if(!this.validateService.validatePassword(user.password)) {
      this.password = undefined;
      this.confirmPwd = undefined;
      this.flashMessage.show("Le mot de passe doit être compris entre 8 et 20 caractères\n et avoir au moins un chiffre une majuscule et une minuscule", {cssClass: this.validateService.flashMsgClass, timeout: 8000});
      return false;
    }

    if(this.password != this.confirmPwd) {
      this.password = undefined;
      this.confirmPwd = undefined;
      this.flashMessage.show("Les mots de passe ne correspondent pas", {cssClass: this.validateService.flashMsgClass, timeout:5000});
      return false;
    }

    this.authService.registerUser(user).subscribe((data, err) => {
      this.errorService.serverError(err, data, () => {
        if(data.success) {
          this.router.navigate(['/login']);
        }
        else {
          this.flashMessage.show(data.msg, {cssClass: this.validateService.flashMsgClass, timeout: 5000});
          this.router.navigate(['/register']);
        }
      });
    });
  }
}
