import { Injectable } from '@angular/core';
import { Http, Headers, ResponseContentType } from '@angular/http';
import { tokenNotExpired } from 'angular2-jwt';
import { Router } from '@angular/router';
import 'rxjs/add/operator/map';

@Injectable()
export class AuthService {
  authToken:any;
  user:any;
  webUrl = "http://localhost:3000/";

  constructor(private http:Http,
              private router:Router) { }

  registerUser(user):any {
    let headers = new Headers();
    headers.append('Content-Type', 'application/json');
    return this.http.post(this.webUrl + 'users/register', user, {headers: headers}).map(res => res.json());
  }

  authenticateUser(user):any {
    let headers = new Headers();
    headers.append('Content-Type', 'application/json');
    return this.http.post(this.webUrl + 'users/authenticate', user, {headers: headers}).map(res => res.json());
  }

  storeUserData(token, user):any {
    localStorage.setItem('id_token', token);
    this.authToken = token;
    this.user = user;
  }

  loadToken() {
    const token = localStorage.getItem('id_token');
    this.authToken = token;
  }

  loggedIn() {
    return tokenNotExpired('id_token');
  }

  logout() {
    this.authToken = null;
    this.user = null;
    localStorage.clear();
  }

  authPost(route, type, data):any {
    if(!this.loggedIn()) {
      this.router.navigate(['/login']);
      return false;
    }
    let headers = new Headers();
    this.loadToken();
    headers.append('Authorization', this.authToken);
    headers.append('Content-Type', type);
    headers.append('Cache-control', 'no-cache');
    return this.http.post(this.webUrl + route, data, {headers: headers}).map(res => res.json());
  }

  authGet(route):any {
    let headers = new Headers();
    this.loadToken();
    headers.append('Authorization', this.authToken);
    headers.append('Content-Type', 'application/json');
    headers.append('Cache-control', 'no-cache');
    return this.http.get(this.webUrl + route, {headers: headers}).map(res => res.json());
  }

  downloadFile(route):any {
    let headers = new Headers();
    this.loadToken();
    headers.append('Authorization', this.authToken);
    headers.append('Content-Type', 'application/json');
    headers.append('Cache-control', 'no-cache');
    let options = {
      responseType: ResponseContentType.Blob,
      headers: headers
    };
    return this.http.get(this.webUrl + route, options).map(res => res.blob());
  }
}
