import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Rx';
import { ConfirmDialogComponent } from '../confirm-dialog/confirm-dialog.component';
import { RenameDialogComponent } from '../rename-dialog/rename-dialog.component';
import { PagelistDialogComponent } from '../pagelist-dialog/pagelist-dialog.component';
import { InfoDialogComponent } from '../info-dialog/info-dialog.component';
import { SliderDialogComponent } from '../slider-dialog/slider-dialog.component';
import { MatDialogRef, MatDialog, MatDialogConfig } from '@angular/material';
import { MathService } from './math.service';

@Injectable()
export class DialogService {

  constructor(private dialog: MatDialog,
              private ms:MathService) { }

  confirm(title, message, isHTML = false): Observable<boolean> {

    let dialogRef: MatDialogRef<ConfirmDialogComponent>;

    dialogRef = this.dialog.open(ConfirmDialogComponent);
    dialogRef.componentInstance.title = title;
    dialogRef.componentInstance.message = message;
    dialogRef.componentInstance.isHTML = isHTML;

    return dialogRef.afterClosed();
  }

  rename(title, message): Observable<string> {
    let dialogRef: MatDialogRef<RenameDialogComponent>;

    dialogRef = this.dialog.open(RenameDialogComponent);
    dialogRef.componentInstance.title = title;
    dialogRef.componentInstance.message = message;

    return dialogRef.afterClosed();
  }

  addToPage(title, pages): Observable<any[]> {
    let dialogRef: MatDialogRef<PagelistDialogComponent>;

    dialogRef = this.dialog.open(PagelistDialogComponent);
    dialogRef.componentInstance.title = title;
    dialogRef.componentInstance.pages = pages;
    return dialogRef.afterClosed();
  }

  info(title, message):any {
    let dialogRef: MatDialogRef<InfoDialogComponent>;

    dialogRef = this.dialog.open(InfoDialogComponent);
    dialogRef.componentInstance.title = title;
    dialogRef.componentInstance.message = message;
  }

  sliderValue(title, min, max, elem): Observable<number> {
    let pos = 0;
    if(elem.type == "T") {
      pos = (this.ms.reverseTimeScale(elem.value)%4);
      pos = Math.floor(pos/4*100);
    }
    else {
      pos = Math.floor((elem.value + 1) * 50);
    }

    let dialogRef: MatDialogRef<SliderDialogComponent>;

    dialogRef = this.dialog.open(SliderDialogComponent);
    dialogRef.componentInstance.title = title;
    dialogRef.componentInstance.sliderValue = elem.value;
    dialogRef.componentInstance.sliderPos = pos;
    dialogRef.componentInstance.sliderElem = elem;
    dialogRef.componentInstance.sliderMin = min;
    dialogRef.componentInstance.sliderMax = max;
    return dialogRef.afterClosed();
  }
}
