const async = require('async');

mass = ['red', 'brt', 'surd1', 'surd2', 'surd3', 'cald', 'distquad', 'distquad2', 'distexp', 'decl'];
module.exports.mass = mass;

ton = ['bharm', 'comb', 'degr', 'dom', 'pcyc2', 'pcyc3', 'pcyc4', 'plog1', 'plog3'];
module.exports.ton = ton;

mod = ['mlfo', 'mtri', 'msqu'];
 module.exports.mod = mod;

module.exports.base = {
  size: [{type: "C", value: null}],
  time: [{type: "T", value: null}],
  amp: [{type: "env", value: [{type: "K", value: null}, {type: "K", value: null}]}, {type: "mod", value: null}],
  freq: [{type: "param", name: "freq", value: [{type: "K/env", value: null}, {type: "mod", value: null}]}],
  mass: [{type: "fonc1", value: null}],
  ton: [{type: "fonc2", value: null}],
  color: [{type: "color", value: null}]
};

module.exports.baseElements = [
  {name: "K", type: "K", quantity: 0, content: null},
  {name: "T", content: [
    {name: "T1", type: "T", quantity: 0, content: null},
    {name: "T2", type: "T", quantity: 0, content: null},
    {name: "T3", type: "T", quantity: 0, content: null},
    {name: "T4", type: "T", quantity: 0, content: null},
    {name: "T5", type: "T", quantity: 0, content: null}
  ]},
  {name: "C", content: [
    {name: 8, type:"C", quantity: 0, content: null},
    {name: 12, type:"C", quantity: 0, content: null},
    {name: 16, type:"C", quantity: 0, content: null},
    {name: 20, type:"C", quantity: 0, content: null},
    {name: 24, type:"C", quantity: 0, content: null},
  ]},
  {name: "env", type: "env", quantity: 0, content: null},
  {name: "mod", content: [
    {name: "mlfo", type: "mod", quantity: 0, param: ["alpha", "freq"], content: null},
    {name: "mtri", type: "mod", quantity: 0, param: ["alpha", "freq"], content: null},
    {name: "msqu", type: "mod", quantity: 0, param: ["alpha", "freq"], content: null}
  ]},
  {name: "fonc1", content: [
    {name: "red", type: "fonc1", quantity: 0, param: ["delta"], content: null},
    {name: "brt", type: "fonc1", quantity: 0, param: ["delta"], content: null},
    {name: "surd1", type: "fonc1", quantity: 0, param: ["alpha"], content: null},
    {name: "surd2", type: "fonc1", quantity: 0, param: ["alpha"], content: null},
    {name: "surd3", type: "fonc1", quantity: 0, param: ["alpha"], content: null},
    {name: "cald", type: "fonc1", quantity: 0, param: ["num"], content: null},
    {name: "distquad", type: "fonc1", quantity: 0, param: ["beta", "delta"], content: null},
    {name: "distquad2", type: "fonc1", quantity: 0, param: ["beta"], content: null},
    {name: "distexp", type: "fonc1", quantity: 0, param: ["beta"], content: null},
    {name: "decl", type: "fonc1", quantity: 0, param: ["delta"], content: null}
  ]},
  {name: "fonc2", content: [
    {name: "bharm", type: "fonc2", quantity: 0, param: ["alpha"], content: null},
    {name: "comb", type: "fonc2", quantity: 0, param: ["delta"], content: null},
    {name: "degr", type: "fonc2", quantity: 0, param: ["beta"], content: null},
    {name: "dom", type: "fonc2", quantity: 0, param: ["alpha", "num"], content: null},
    {name: "pcyc2", type: "fonc2", quantity: 0, param: ["alpha"], content: null},
    {name: "pcyc3", type: "fonc2", quantity: 0, param: ["alpha", "alpha"], content: null},
    {name: "pcyc4", type: "fonc2", quantity: 0, param: ["alpha", "alpha", "alpha"], content: null},
    {name: "plog1", type: "fonc2", quantity: 0, param: ["delta"], content: null},
    {name: "plog3", type: "fonc2", quantity: 0, param: ["delta", "alpha"], content: null}
  ]},
  {name: "col", type: "color", quantity: 0, param: ["alpha", "freq", "delta"], content: null}
];

module.exports.generateFormula = function(content, callback) {
  let arrayList = {};
  let formula = "(sonify ";

  async.forEach(Object.keys(content), (c, callb) => {
    if(Array.isArray(content[c])) {
      let formularray = new Array();
      recFormula(content[c], formularray, (f) => {
        if(c == "size") {
          arrayList[c] = content.size[0].value;
        }
        else if(c == "freq") {
          arrayList[c] = f[0]; //only 1 element. array is used for easier front-end display
        }
        else if(c == "amp") {
          arrayList[c] = sortFunction(f, this.mod, true);
        }
        else {
          arrayList[c] = f;
        }
        callb();
      });
    }
    else {
      arrayList[c] = content[c];
      callb();
    }
  }, () => {
    arrayList.mass = sortFunction(arrayList.mass , this.mass);
    arrayList.ton = sortFunction(arrayList.ton , this.ton);
    //create the final formula string
    //can not use loop because collection is not ordered
    formula += JSON.stringify(arrayList.size) + " ";
    formula += JSON.stringify(arrayList.time) + " ";
    formula += JSON.stringify(arrayList.amp) + " ";
    formula += JSON.stringify(arrayList.freq) + " ";
    formula += JSON.stringify(arrayList.mass) + " ";
    formula += JSON.stringify(arrayList.ton) + " ";
    formula += JSON.stringify(arrayList.color) + ")";
    formula = formula.replace(/\"/g, "");
    formula = formula.replace(/,/g, " ");
    callback(formula);
  });
}

//go through each tree to get only the "value" component of the data structure
function recFormula(arr, formula, callback) {
  async.forEach(arr, (i, cb) => {
    if(Array.isArray(i.value)) {
      let formularray = new Array();
      recFormula(i.value, formularray, (f) => {
        if(f.length > 0 && i.type == "param") {
          f = sortFunction(f, this.mod, true);
          formula.push(f);
        }
        else if(f.length > 0) {
          formula.push(f);
        }
        cb();
      });
    }
    else if(i.value != null) {
      formula.push(i.value);
      cb();
    }
    else if(i.value == null && i.type == "K/nil") {
      formula.push("nil");
      cb();
    }
    else {
      cb();
    }
  }, () => {
    callback(formula);
  });
}

//mass function need to be ordered
//used for all functions to limit formula duplication
function sortFunction(arr, funcList, isMod = false) {
  let sorted = new Array();
  // push the env or K/env before mod list
  if(isMod) {
    sorted.push(arr[0]);
  }
  for(let ref of funcList) {
    for(let a of arr) {
      if(ref == a[0]) {
        sorted.push(a);
      }
    }
  }
  return sorted;
}
