const Arcane = require('../models/arcane');
const User = require('../models/user');
const async = require('async');
const fs = require('fs');
const Formula = require('../config/formula');
const tools = require('../tools/tools');
const udp = require('../supercollider/udp');
const path = require('path');
const pubsub = require('pubsub-js');



let schema = {};

let content = fs.readFileSync('random.txt', 'utf8');
content = content.split("\n");
content.pop(); //remove the last empty element
let count = content.length;

let username = 'Base';
let newUser = new User(username, 'Base@mail.com', 'Base123Base');

udp.open();
udp.on('ready', () => {
  console.log("Processing...");
  User.addUser(newUser, (err, result) => {
    if(err == null && result.success) {
      let inter = setInterval(() => {
        let c = content[count-1];
        let parsed = replaceName(c);
        parsed = parseFormula(parsed);

        //create schema
        let schema = {};
        schema.size = [{type: "C", value: Number(parsed[1])}];

        let parsedParam = parsed[2].replace(/[ ]/g, ',');
        schema.time = parseParam(JSON.parse(parsedParam), "time");

        parsedParam = parsed[3].replace(/[ ]/g, ',');
        schema.amp = parseParam(JSON.parse(parsedParam), "amp");

        parsedParam = parsed[4].replace(/[ ]/g, ',');
        schema.freq = parseParam(JSON.parse(parsedParam), "freq");

        parsedParam = parsed[5].replace(/[ ]/g, ',');
        schema.mass = parseParam(JSON.parse(parsedParam), "mass");

        parsedParam = parsed[6].replace(/[ ]/g, ',');
        schema.ton = parseParam(JSON.parse(parsedParam), "ton");

        parsedParam = parsed[7].replace(/[ ]/g, ',');
        schema.color = parseParam(JSON.parse(parsedParam), "color");

        let fileName = username + String(Date.now());
        let filePath = path.join(__dirname, "../sounds/" + fileName + ".WAV");
        let used = tools.getElementsOf(schema);
        const cost = tools.elementsCost(used);
        const newArcane = new Arcane(username, fileName, c, schema, cost);

        let msg = {
          address: "/sonifyNode",
          args: [
            {type: "s", value: username},
            {type: "s", value: filePath},
            {type: "s", value: c},
          ]
        };
        udp.send(msg);

        let token = pubsub.subscribe(username, (pubName, data) => {
          pubsub.unsubscribe(username);
          if(data != 0) {
            console.log("SC-Sonify-FLSC ERROR");
          }
          else {
            Arcane.createArcane(newArcane, (err, arcaneId) => {
              if(err) {
                console.log('ERROR : arcane registration failed');
              }
              else {
                createArcaneImage(newArcane.fileName + ".svg", (err) => {
                  if(err) {
                    console.log('ERROR : failed creating arcane image');
                  }
                });
              }
            });
          }
        });
        console.log("arcane : " + count + " / " + content.length);
        count --;
        if(count <= 0){
          clearInterval(inter);
        }
      }, 1000);
    }
  });
});


//########### UDP #####################################


udp.on("error", function (err) {
  console.log("OSC error : " + err);
});

udp.on('message', (msg) => {
  const data = JSON.parse(JSON.stringify(msg));
  pubsub.publish(data.args[0].value, data.args[1].value);
});



//############## PARSE ##############################

function parseFormula(formula) {
  formula = formula.replace('(', '');
  let s = '';
  let array = [];
  let index = 0;
  for(let f of formula) {
    s += f;
    if(f == '[') {
      index ++;
    }
    if(f == ']') {
      index --;
    }
    if((f == ' ' || f == ')') && index == 0) {
      array.push(s.substring(0, s.length - 1));
      s = '';
    }
  }

  return array;
}

//string to array for each param of the formula
function parseParam(arr, type) {
  let res = [];
  switch (type) {
    case "time":
    for(let a of arr) {
      res.push({type: "T", name: timeValue(Number(a)), value: Number(a)});
    }
    res.push({type: "T", value: null});
    break;

    case "amp":
      res.push(parseType(arr[0], "env"));
      if(arr.length > 1) {
        for(let i=1; i<arr.length; i++) {
          res.push(parseType(arr[i], "mod"));
        }
      }
      else {
        res.push({type: "mod", value: null});
      }
    break;

    case "freq":
      res.push({type: "param", name:"freq", value:parseType(arr, "param")});
    break;

    case "mass":
      if(arr.length > 0) {
        for(let a of arr) {
          res.push(parseType(a, "fonc1"));
        }
      }
      res.push({type: "fonc1", value: null});
    break;

    case "ton":
      if(arr.length > 0) {
        for(let a of arr) {
          res.push(parseType(a, "fonc2"));
        }
      }
      res.push({type: "fonc2", value: null});
    break;

    case "color":
      if(arr.length > 0) {
        for(let a of arr) {
          res.push(parseType(a, "color"));
        }
      }
      res.push({type: "color", value: null});
    break;

    default:
    break;
  }
  return res;
}

//get which time element is used
function timeValue(t) {
  res = 0;
  if(t == 0.002) {
    res = 0;
  }
  else {
    res = Math.sqrt(t/0.01);
  }
  Math.floor(res);
  return 'T' + (Math.floor(res/4)+1);
}


function parseType(arr, type) {
  let res = {};
  switch (type) {
    case "env":
      res = {type: "env", value: []};
      res.value.push({type: "K", value: arr[0]});
      for(let i=1; i<arr.length-1; i++) {
        let val = arr[i];
        if(arr[i] == "nil") {
          val = null;
        }
        res.value.push({type: "K/nil", value: val});
      }
      res.value.push({type: "K", value: arr[arr.length-1]});
      return res;

    case "mod":
      res = {type: "mod", value: []};
      res.value.push({type: "M", value: arr[0]});
      for(let i=1; i < arr.length; i++) {
        for(let m of Formula.baseElements[4].content) {
          if(m.name == arr[0]) {
            res.value.push({type: "param", name:m.param[i-1], value: parseType(arr[i], "param")});
          }
        }
      }
      return res;

    case "param":
      res = [];
      if(arr.length > 1) {
        res.push(parseType(arr[0], "K/env"));
        for(let i=1; i<arr.length; i++) {
          res.push(parseType(arr[i], "mod"));
        }
      }
      else {
        res.push(parseType(arr[0], "K/env"));
      }
      res.push({type: "mod", value: null});
      return res;


    case "K/env":
      if(Array.isArray(arr)) {
        res = parseType(arr, "env");
        res.type = "K/env";
        return res;
      }
      else {
        return {type: "K/env", value: arr};
      }

    case "fonc1":
      res = {type: "fonc1", value: []};
      res.value.push({type: "F1", value: arr[0]});
      for(let i=1; i < arr.length; i++) {
        for(let m of Formula.baseElements[5].content) {
          if(m.name == arr[0]) {
            res.value.push({type: "param", name:m.param[i-1], value: parseType(arr[i], "param")});
          }
        }
      }
      return res;

    case "fonc2":
      res = {type: "fonc2", value: []};
      res.value.push({type: "F2", value: arr[0]});
      for(let i=1; i < arr.length; i++) {
        for(let m of Formula.baseElements[6].content) {
          if(m.name == arr[0]) {
            res.value.push({type: "param", name:m.param[i-1], value: parseType(arr[i], "param")});
          }
        }
      }
      return res;

    case "color":
      res = {type: "color", value: []};
      res.value.push({type: "col", value: arr[0]});
      for(let i=1; i < arr.length; i++) {
        let m = Formula.baseElements[7];
        if(m.name == arr[0]) {
          res.value.push({type: "param", name:m.param[i-1], value: parseType(arr[i], "param")});
        }
      }
      return res;

    default:
    return null;
  }
}

function replaceName(str) {
  let re = null;
  for(let e of Formula.mod) {
    re = new RegExp(e, "g");
    str = str.replace(re, '"' + e + '"');
  }
  for(let e of Formula.mass) {
    if(e == "distquad") {
      e = "distquad ";
      re = new RegExp(e, "g");
      str = str.replace(re, '"distquad" ');
    }
    else {
      re = new RegExp(e, "g");
      str = str.replace(re, '"' + e + '"');
    }
  }
  for(let e of Formula.ton) {
    re = new RegExp(e, "g");
    str = str.replace(re, '"' + e + '"');
  }
  re = new RegExp("col", "g");
  str = str.replace(re, '"col"');
  re = new RegExp("nil", "g");
  str = str.replace(re, '"nil"');
  return str;
}

//##################### CREATE IMAGE ########################

function pointOnCircle(angle, radius, center) {
  let rad = angle * Math.PI / 180;
  let x = radius * Math.cos(rad) + center.x;
  let y = radius * Math.sin(rad) + center.y;
  return {x: x, y: y};
}

function createArcaneImage(fileName, callback) {
  let data = "<svg xmlns='http://www.w3.org/2000/svg' version='1.0' width='200' height='200'>";

  const div = 7;
  const edge = 360/(div*4);
  const center = {x: 100, y: 100};
  const radius1 = 80, radius2 = 40;
  let p;

  //draw the triangles
  data += "<polygon points='";
  for(let i=0; i < div; i++) {
    p = pointOnCircle(((360/div * i) - edge), radius2, center);
    data += p.x + "," + p.y + " ";
    p = pointOnCircle(((360/div * i) + edge), radius1, center);
    data += p.x + "," + p.y + " ";
    p = pointOnCircle(((360/div * (i+1)) - edge), radius2, center);
    data += p.x + "," + p.y + " ";

    data += "' fill='rgb(" + tools.randi(256) + ", " + tools.randi(256) + ", " + tools.randi(256) + ")'/>";
    if(i+1 < div) {
      data += "<polygon points='";
    }
  }

  data += "<circle cx='100' cy='100' r='30' fill='rgb(" + tools.randi(256) + ", " + tools.randi(256) + ", " + tools.randi(256) + ")' />"
  data += "</svg>";

  fileName = path.join(__dirname, "../arcaneImages/", fileName);
  fs.writeFile(fileName, data, (err) => {
    callback(err);
  });
}
