const db = require('../db');
const sem = require('semaphore')(1);
const mem = require('./mem');
const Arcane = require('./arcane');
const tools = require('../tools/tools');

function Page(name, author, content, sim, isStatic) {
  this.name = name;
  this.author = author;
  this.content = content;
  this.sim = sim;
  this.isStatic = isStatic;
  this.id = -1;
}

module.exports = Page;

module.exports.getPage = function(id, callback) {
  sem.take(() => {
    let sql = "SELECT * FROM page WHERE id = " + id + "";
    db.query(sql, (err, rows, fields) => {
      if(err) {
        sem.leave();
        callback(err, undefined);
      }
      else if(rows.length > 0) {
        let p = new Page(rows[0].name, rows[0].author, JSON.parse(rows[0].content), JSON.parse(rows[0].sim), rows[0].isStatic);
        p.id = rows[0].id;
        sem.leave();
        callback(null, p);
      }
      else {
        sem.leave();
        callback("ERROR: getPage: no page found", undefined);
      }
    });
  });
}

module.exports.getPageByName = function(name, author, callback) {
  sem.take(() => {
    let sql = "SELECT * FROM page WHERE name='" + name + "' AND author='" + author + "'";
    db.query(sql, (err, rows, fields) => {
      if(err) {
        sem.leave();
        callback(err, undefined);
      }
      else if(rows) {
        let p = new Page(rows[0].name, rows[0].author, JSON.parse(rows[0].content), JSON.parse(rows[0].sim), rows[0].isStatic);
        p.id = rows[0].id;
        sem.leave();
        callback(null, p);
      }
      else {
        sem.leave();
        callback("ERROR: getPageByName: no page found", undefined);
      }
    });
  });
}

module.exports.getAllPageFrom = function(author, callback) {
  sem.take(() => {
    let sql = "SELECT * FROM page WHERE author='" + author + "'";
    db.query(sql, (err, rows, fields) => {
      if(err) {
        sem.leave();
        callback(err, undefined);
      }
      else if(rows) {
        let arr = new Array();
        for(let r of rows) {
          let p = new Page(r.name, r.author, JSON.parse(r.content), JSON.parse(r.sim), r.isStatic);
          p.id = r.id;
          arr.push(p);
        }
        sem.leave();
        callback(null, arr);
      }
      else {
        sem.leave();
        callback("ERROR: getAllPageFrom: no page found", undefined);
      }
    });
  });

}

module.exports.createPage = function(name, author, isStatic, callback) {
  sem.take(() => {
    const content = [];
    const sim = {};
    sql = "INSERT INTO page (name, author, content, sim, isStatic) VALUES ('" + name +
    "', '" + author +
    "', '" + JSON.stringify(content) +
    "', '" + JSON.stringify(sim) +
    "', '" + isStatic + "')";
    db.query(sql, (err, result) => {
      res = new Page(name, author, content, sim, isStatic);
      res.id = result.insertId;
      sem.leave();
      callback(err, res);
    });
  });
}

module.exports.deletePage = function(id, callback) {
  sem.take(() => {
    let sql = "DELETE FROM page WHERE id = '" + id + "'";
    db.query(sql, (err, result) => {
      sem.leave();
      callback(err);
    });
  });
}

module.exports.addArcane = function(pageId, arcaneId, callback) {
  this.getPage(pageId, (err, page) => {
    if(err) {
      callback(err);
    }
    else {
      page.content.push(arcaneId);
      let sql = "UPDATE page SET content = '" + JSON.stringify(page.content) + "' WHERE id = '" + pageId + "'";
      db.query(sql, (err, result) => {
        callback(err);
      });
    }
  });
}

module.exports.removeArcane = function(pageId, arcaneId, callback) {
  this.getPage(pageId, (err, page) => {
    if(err) {
      callback(err);
    }
    else {
      let index = page.content.indexOf(arcaneId);
      if (index > -1) {
        page.content.splice(index, 1);
      }
      let sql = "UPDATE page SET content = '" + JSON.stringify(page.content) + "' WHERE id = '" + pageId + "'";
      db.query(sql, (err, result) => {
        callback(err);
      });
    }
  });
}

module.exports.isInPage = function(pageId, arcaneId, callback) {
  this.getPage(pageId, (err, page) => {
    if(err) {
      callback(err, undefined);
    }
    else {
      let isIn = false;
      for(let c of page.content) {
        if(c == arcaneId) {
          isIn = true;
          break;
        }
      }
      callback(err, isIn);
    }
  });
}

module.exports.updatePage = function(field, data, id, callback) {
  sem.take(() => {
    let sql = "UPDATE page SET " + field + " = '" + data + "' WHERE id = '" + id + "'";
    db.query(sql, (err) => {
      sem.leave();
      callback(err);
    });
  });
}

module.exports.updatePages = function(field, data, ids, callback) {
  sem.take(() => {
    let sql = "UPDATE page SET " + field + " = CASE ";
    if(!Array.isArray(ids)) {
      sql += "WHEN id = " + ids + " THEN '" + JSON.stringify(data[0]) + "' ";
    }
    for(let i=0; i<ids.length; i++) {
      sql += "WHEN id = " + ids[i] + " THEN '" + JSON.stringify(data[i]) + "' ";
    }
    sql += "ELSE " + field + " END;";

    db.query(sql, (err) => {
      sem.leave();
      callback(err);
    });
  });
}


//####################### RECOMMENDATION #############################


const MAX_RECOMMENDATION = 3;

module.exports.recommendation = function(id, user, callback) {
  this.getPageByName("Arcanes connus", user.username, (err, p) => {
    if(err) {
      callback(err, undefined);
    }
    else if(p.id == id) {
      callback(null, undefined);
    }
    else {
      //remove known arcane from recommendation possibilities
      Arcane.getAllExceptList(p.content, (err, arcanes) => {
        if(err) {
          callback(err, undefined);
        }
        //if unknown arcanes are found
        else if(arcanes.length > 0) {
          mem.getArcanesOfPage(id, (err, pageMem) => {
            if(err) {
              callback(err, null);
            }
            else {
              //remove already refused recommendation
              if(pageMem) {
                for(let m of pageMem) {
                  if(m.value == -1) {
                    arcanes.splice(arcanes.findIndex(a => a.id == m.arcane), 1);
                  }
                }
              }

              if(arcanes.length > 0) {
                let arcaneIds = arcanes.map(a => a.id);
                this.getPage(id, (err, page) => {
                  let simKeys = Object.keys(page.sim);
                  if(err) {
                    callback(err);
                  }
                  else if(simKeys.length > 0) {
                    mem.getCommendableMems(arcaneIds, simKeys, (err, mems) => {
                      if(err) {
                        callback(err);
                      }
                      else if(mems) {
                        let prob = new Array();
                        let tmpId = -1;
                        let i = -1;
                        //compute prob for each arcanes in similar other pages
                        for(let m of mems) {
                          if(tmpId != m.arcane) {
                            tmpId = m.arcane;
                            prob.push({id: tmpId, value: 0});
                            i++;
                          }
                          prob[i].value += m.value * page.sim[m.page];
                        }
                        prob = getBetterRecommendation(prob);
                        //get arcanes from prob
                        let recArcane = new Array();
                        for(p of prob) {
                          if(p.value >= 0) {
                            recArcane.push(arcanes.find(a => {return a.id == p.id;}));
                          }
                        }

                        if(recArcane.length < MAX_RECOMMENDATION) {
                          //prevent duplication with random
                          for(let ra of recArcane) {
                            arcanes.splice(arcanes.findIndex(a => a.id == ra.id), 1);
                          }
                          let r = randomRecommendation(arcanes, MAX_RECOMMENDATION - recArcane.length);
                          recArcane = recArcane.concat(r);
                        }
                        callback(null, recArcane);

                      }
                      else {
                        //no commendable arcanes
                        let r = randomRecommendation(arcanes, MAX_RECOMMENDATION);
                        callback(null, r);
                      }
                    });
                  }
                  else {
                    //no similarity with other pages
                    let r = randomRecommendation(arcanes, MAX_RECOMMENDATION);
                    callback(null, r);
                  }
                });
              }
              else {
                //no unknown arcane available (all refused)
                callback(null, undefined);
              }
            }
          });
        }
        else {
          //no unknown arcanes
          callback(null, undefined);
        }
      });
    }
  });
}

function randomRecommendation(arcanes, n) {
  let index = 0;
  let r = new Array();
  for(let i=0; i < n; i++) {
    if(arcanes.length > 0) {
      index = tools.randi(arcanes.length);
      r.push(arcanes[index]);
      arcanes.splice(index, 1);
    }
    else {
      break;
    }
  }
  return r;
}

function getBetterRecommendation(prob) {
  let j = 0;
  let max = 0;
  let iMax = 0;
  let sorted = new Array();
  while(j < MAX_RECOMMENDATION && prob.length > 0) {
    max = prob[0];
    for(let i=1; i < prob.length; i++) {
      if(prob[i].value > max) {
        max = prob[i].value;
        iMax = i;
      }
    }
    sorted.push(max);
    prob.splice(iMax, 1);
    j++;
  }
  return sorted;
}


//###################### SIMILARITY ###############################


module.exports.updateSimilarity = function(a, pageId, arcaneId, callback) {
  mem.getMem(arcaneId, pageId, (err, mv) => {
    if(err) {
      callback(err);
    }
    else {
      //difference between previous and next value
      a -= mv;
      //get all page id and membership values for pages containing this arcane
      mem.getPagesWithArcane(arcaneId, (err, memPage) => {
        if(err) {
          callback(err);
        }
        else {
          //first add of this arcane in a page
          if(!memPage) {
            mem.createMem(arcaneId, pageId, a+mv, (err) => {
              callback(err);
            });
          }
          else {
            this.getPage(pageId, (err, p) => {
              if(err) {
                callback(err);
              }
              else {
                let memKeys = Object.keys(memPage);
                //get all page similarity of the pages containing the arcane
                this.getPagesSim(memKeys, (err, pagesSim) => {
                  if(err) {
                    callback(err);
                  }
                  else {
                    //compute simimlarity of the page receiving the arcane with other pages containing it
                    //and update opposite page aswell
                    let i = 0;
                    for(let m in memPage) {
                      if(m != p.id) {
                        if(!p.sim[m]) {
                          p.sim[m] = a*memPage[m];
                        }
                        else {
                          p.sim[m] += a*memPage[m];
                        }
                        let opposite = pagesSim[i];
                        if(!opposite[p.id]) {
                          opposite[p.id] = a*memPage[m];
                        }
                        else {
                          opposite[p.id] += a*memPage[m];
                        }
                      }
                      i++;
                    }
                    //store changes
                    //this update could be merge with the next one
                    this.updatePages('sim', pagesSim, memKeys, (err) => {
                      if(err) {
                        callback(err);
                      }
                      else {
                        this.updatePage('sim', JSON.stringify(p.sim), p.id, (err) => {
                          if(err) {
                            callback(err);
                          }
                          else {
                            //arcane added to page
                            if(a == 1) {
                              mem.createMem(arcaneId, pageId, 1, (err) => {
                                callback(err);
                              });
                            }
                            //when a recommendation is refused then added
                            else if(a == 2) {
                              mem.updateValue(arcaneId, pageId, 1, (err) => {
                                callback(err);
                              });
                            }
                            //recommendation refused
                            else if(a == -1 && mv == 0) {
                              mem.createMem(arcaneId, pageId, -1, (err) => {
                                callback(err);
                              });
                            }
                            //arcane removed from page
                            else {
                              mem.deleteMem(arcaneId, pageId, (err) => {
                                callback(err);
                              });
                            }
                          }
                        });
                      }
                    });
                  }
                });
              }
            });
          }
        }
      });
    }
  });
}

module.exports.getPagesSim = function(pageId, callback) {
  sem.take(() => {
    let sql = "SELECT sim FROM page WHERE id IN (" + pageId + ") ORDER BY FIND_IN_SET(id, '" + pageId + "')";
    db.query(sql, (err, rows, fields) => {
      if(err) {
        sem.leave();
        callback(err, undefined);
      }
      else if(rows.length > 0) {
        let sims = new Array();
        for(let r of rows) {
          sims.push(JSON.parse(r.sim));
        }
        sem.leave();
        callback(null, sims);
      }
      else {
        sem.leave();
        callback(null, null);
      }
    });
  });
}

module.exports.removeSimilarity = function(pageId, callback) {
  this.getPage(pageId, (err, p) => {
    if(err) {
      callback(err);
    }
    else {
      //if p.sim empty then nothing to remove
      let isEmpty = true;
      for(let i in p.sim) {
        if(p.sim.hasOwnProperty(i)) {
          isEmpty = false;
          break;
        }
      }
      if(isEmpty) {
        callback(null);
      }
      else {
        //get all pages where the deleted page appears then delete its reference
        this.getPagesSim(Object.keys(p.sim), (err, sims) => {
          if(err) {
            callback(err);
          }
          else {
            for(let s of sims) {
              delete s[pageId];
            }
            this.updatePages('sim', sims, Object.keys(p.sim), (err) => {
              callback(err);
            });
          }
        });
      }
    }
  });
}
