const bcrypt = require('bcryptjs');
const db = require('../db');
const Page = require('./page');
const Formula = require('../config/formula');
const sem = require('semaphore')(1);

//user object
function User(username, email, password) {
  this.username = username;
  this.email = email;
  this.password = password;
  this.id = -1;
}

module.exports = User;

module.exports.MAX_AURA = 1000000;

//add user to db
module.exports.addUser = function(newUser, callback) {
  bcrypt.genSalt(10, (err, salt) => {
    if(err) {
      callback(err, null);
    }
    else {
      //crypting password
      bcrypt.hash(newUser.password, salt, (err, hash) => {
        if(err) {
          callback(err, null);
        }
        else {
          //check if user is already registered
          let sql = "SELECT * FROM user WHERE username = '" + newUser.username + "'";
          db.query(sql, (err, rows, fields) => {
            if(err) {
              callback(err, null);
            }
            else if(rows[0]) {
              callback(err, {success: false, msg: "Un utilisateur est déjà enregistré à ce nom"});
            }
            //check if email is already used
            else {
              sql = "SELECT * FROM user WHERE email = '" + newUser.email + "'";
              db.query(sql, (err, rows, fields) => {
                if(err) {
                  callback(err, null);
                }
                else if(rows[0]) {
                  callback(err, {success: false, msg: "Un utilisateur est déjà enregistré avec cette adresse e-mail"});
                }
                //then add new user in database
                else {
                  newUser.password = hash;
                  Page.createPage('Arcanes connus', newUser.username, 1, (err, p) => {
                    if(err) {
                      callback(err, undefined);
                    }
                    else {
                      const pages = [{ name:"Arcanes connus", id: p.id }];
                      const emptyArray = [];
                      sql = "INSERT INTO user (username, email, password, pages, workspaces, elements, aura) VALUES ('" + newUser.username +
                        "', '" + newUser.email +
                        "', '" + newUser.password +
                        "', '" + JSON.stringify(pages) +
                        "', '" + JSON.stringify(emptyArray) +
                        "', '" + JSON.stringify(Formula.baseElements) +
                        "', '" + this.MAX_AURA + "')";
                      db.query(sql, (err, result) =>{
                        callback(err, {success:true});
                      });
                    }
                  });
                }
              });
            }
          });
        }
      });
    }
  });
}

module.exports.getUser = function(field, data, callback) {
  let sql = "SELECT * FROM user WHERE " + field + " = '" + data + "'";
  db.query(sql, (err, rows, fields) => {
    if(err) {
      callback(err, undefined);
    }
    else if(rows[0]){
      let user = new User(rows[0].username, rows[0].email, rows[0].password);
      user.id = rows[0].id;
      callback(null, user);
    }
    else {
      callback(null, undefined);
    }
  });
}

module.exports.getUserPages = function(id, callback) {
  let sql = "SELECT pages FROM user WHERE id = '" + id + "'";
  db.query(sql, (err, rows, fields) => {
    if(err) {
      callback(err, undefined);
    }
    else if(rows[0]) {
      callback(err, JSON.parse(rows[0].pages));
    }
    else {
      callback(null, undefined);
    }
  });
}

module.exports.getUserWorkspaces = function(id, callback) {
  let sql = "SELECT workspaces FROM user WHERE id = '" + id + "'";
  db.query(sql, (err, rows, fields) => {
    if(err) {
      callback(err, undefined);
    }
    else if(rows[0]) {
      callback(err, JSON.parse(rows[0].workspaces));
    }
    else {
      callback(null, undefined);
    }
  });
}

module.exports.getUserElements = function(id, callback) {
  let sql = "SELECT elements FROM user WHERE id = '" + id + "'";
  db.query(sql, (err, rows, fields) => {
    if(err) {
      callback(err, undefined);
    }
    else if(rows[0]) {
      callback(err, JSON.parse(rows[0].elements));
    }
    else {
      callback("ERROR : getUserElements : no object found", undefined);
    }
  });
}

module.exports.getUserAura = function(id, callback) {
  let sql = "SELECT aura FROM user WHERE id= '" + id + "'";
  db.query(sql, (err, rows, fields) => {
    callback(err, rows[0].aura);
  });
}

//used to prevent multiple access when refill aura
module.exports.updateAura = function(id, sumVal, callback) {
  sem.take(() => {
    this.getUserAura(id, (err, aura) => {
      if(err) {
        sem.leave();
        callback(err, undefined);
      }
      else {
        aura += sumVal;
        if(aura > this.MAX_AURA) {
          aura = this.MAX_AURA;
        }
        let sql = "UPDATE user SET aura = '" + aura + "' WHERE id = '" + id + "'";
        db.query(sql, (err) => {
          sem.leave();
          callback(err, aura);
        });
      }
    });
  });
}

module.exports.updateUser = function(field, data, id, callback) {
  let sql = "UPDATE user SET " + field + " = '" + data + "' WHERE id = '" + id + "'";
  db.query(sql, (err) => {
    callback(err);
  });
}

module.exports.addWorkspace = function(idUser, newWorkspace, callback) {
  this.getUserWorkspaces(idUser, (err, workspaces) => {
    if(err) {
      callback(err);
    }
    else {
      workspaces.push(newWorkspace);
      this.updateUser('workspaces', JSON.stringify(workspaces), idUser, (err) => {
        callback(err);
      });
    }
  });
}

module.exports.addPage = function(idUser, newPage, callback) {
  this.getUserPages(idUser, (err, pages) => {
    if(err) {
      callback(err);
    }
    else {
      pages.push(newPage);
      this.updateUser('pages', JSON.stringify(pages), idUser, (err) => {
        callback(err);
      });
    }
  });
}

//compare passwords when user log in
module.exports.comparePassword = function(candidate, hash, callback) {
  bcrypt.compare(candidate, hash, (err, isMatch) => {
    callback(err, isMatch);
  });
}

module.exports.updateAllUsersAura = function(callback) {
  sem.take(() => {
    let sql = "SELECT id, aura FROM user";
    db.query(sql, (err, rows, fields) => {
      if(err) {
        sem.leave();
        callback(err);
      }
      else if(rows.length > 0) {
        sql = "UPDATE user SET aura = CASE ";
        for(r of rows) {
          r.aura += Math.floor((this.MAX_AURA - r.aura) * 0.1);
          if(r.aura > this.MAX_AURA) {
            r.aura = this.MAX_AURA;
          }
          sql += "WHEN id = " + r.id + " THEN '" + r.aura + "' ";
        }
        sql += "ELSE aura END;";

        db.query(sql, (err) => {
          sem.leave();
          callback(err);
        });
      }
      else {
        sem.leave();
        callback(null);
      }
    });
  });
}
