const express = require('express');
const passport = require('passport');
const jwt = require('jsonwebtoken');
const async = require('async');
const pubsub = require('pubsub-js');
const path = require('path');
const fs = require('fs');

const routeError = require('../tools/tools').routeError;
const tools = require('../tools/tools');
const udp = require('../supercollider/udp');
const User = require('../models/user');
const Arcane = require('../models/arcane');
const Formula = require('../config/formula');
const Page = require('../models/page');

const router = express.Router();

module.exports = router;

router.post('/listen', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  let filePath = path.join(__dirname, "../sounds/tmp/" + req.user.username + ".WAV");

  Formula.generateFormula(req.body.content, (formula) => {
    let msg = {
      address: "/sonifyNode",
      args: [
        {type: "s", value: req.user.username},
        {type: "s", value: filePath},
        {type: "s", value: formula},
      ]
    };
    udp.send(msg);
    let token = pubsub.subscribe(req.user.username, (pubName, data) => {
      pubsub.unsubscribe(req.user.username);
      if(data == 0) {
        res.json({success: true});
      }
      else {
        res.json({success: false, msg: "Erreur lors de la création du son veillez réessayer"});
      }
    });
  });
});

router.post('/', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  let fileName = req.user.username + String(Date.now());
  let filePath = path.join(__dirname, "../sounds/" + fileName + ".WAV");
  let used = tools.getElementsOf(req.body.content);
  const cost = tools.elementsCost(used);

  User.getUserAura(req.user.id, (err, aura) => {
    routeError(err, res, () => {
      if(aura - cost < 0) {
        res.json({success: false, msg: "Le coût de cette formule est supérieur à votre aura.\ncoût: " + cost + ""});
      }
      else {
        Formula.generateFormula(req.body.content, (formula) => {
          //OSC message for supercollider
          let msg = {
            address: "/sonifyNode",
            args: [
              {type: "s", value: req.user.username},
              {type: "s", value: filePath},
              {type: "s", value: formula},
            ]
          };
          const newArcane = new Arcane(req.user.username, fileName, formula, req.body.content, cost);

          //test if this formula already exist
          Arcane.getArcane('formula', newArcane.formula, (err, arcane) => {
            routeError(err, res, () => {
              if(arcane) {
                Page.getPageByName("Arcanes connus", req.user.username, (err, known) => {
                  routeError(err, res, () => {
                    //arcane already known
                    if(known.content.includes(arcane.id)) {
                      res.json({success: false, msg: "Vous connaissez déjà cette formule."});
                    }
                    else {
                      //arcane create by another user
                      //add it to this user known arcanes
                      tools.addArcaneToPage("Arcanes connus", req.user, arcane.id, (err) => {
                        routeError(err, res, () => {
                          res.json({success: false, msg: "Cet arcane à déjà été créé par un autre joueur.\n Il sera ajouté à vos arcanes connus."});
                        });
                      });
                    }
                  });
                });
              }
              else {
                //if the formula is not in the database send a message to supercollider to create the wav file
                udp.send(msg);
                //wait for SC response
                let token = pubsub.subscribe(req.user.username, (pubName, data) => {
                  pubsub.unsubscribe(req.user.username);
                  if(data != 0) {
                    res.json({success: false, msg: "Erreur lors de la création du son veillez réessayer"});
                  }
                  else {
                    Arcane.createArcane(newArcane, (err, arcaneId) => {
                      routeError(err, res, () => {
                        createArcaneImage(newArcane.fileName + ".svg", (err) => {
                          routeError(err, res, () => {
                            tools.addArcaneToPage("Arcanes connus", req.user, arcaneId, (err) => {
                              routeError(err, res, () => {
                                User.getUserElements(req.user.id, (err, elem) => {
                                  routeError(err, res, () => {
                                    tools.computeElemQuantity(elem, used, -1);
                                    User.updateUser('elements', JSON.stringify(elem), req.user.id, (err) => {
                                      routeError(err, res, () => {
                                        User.updateAura(req.user.id, -cost, (err, newAura) => {
                                          routeError(err, res, () => {
                                            tools.computeElemQuantity(elem, used, -1);
                                            res.json({success: true, aura: newAura, elements: elem});
                                          });
                                        });
                                      });
                                    });
                                  });
                                });
                              });
                            });
                          });
                        });
                      });
                    });
                  }
                });
              }
            });
          });
        });
      }
    });
  });
});


//#################################################################


function pointOnCircle(angle, radius, center) {
  let rad = angle * Math.PI / 180;
  let x = radius * Math.cos(rad) + center.x;
  let y = radius * Math.sin(rad) + center.y;
  return {x: x, y: y};
}

function createArcaneImage(fileName, callback) {
  let data = "<svg xmlns='http://www.w3.org/2000/svg' version='1.0' width='200' height='200'>";

  const div = 7;
  const edge = 360/(div*4);
  const center = {x: 100, y: 100};
  const radius1 = 80, radius2 = 40;
  let p;

  //draw the triangles
  data += "<polygon points='";
  for(let i=0; i < div; i++) {
    p = pointOnCircle(((360/div * i) - edge), radius2, center);
    data += p.x + "," + p.y + " ";
    p = pointOnCircle(((360/div * i) + edge), radius1, center);
    data += p.x + "," + p.y + " ";
    p = pointOnCircle(((360/div * (i+1)) - edge), radius2, center);
    data += p.x + "," + p.y + " ";

    data += "' fill='rgb(" + tools.randi(256) + ", " + tools.randi(256) + ", " + tools.randi(256) + ")'/>";
    if(i+1 < div) {
      data += "<polygon points='";
    }
  }

  data += "<circle cx='100' cy='100' r='30' fill='rgb(" + tools.randi(256) + ", " + tools.randi(256) + ", " + tools.randi(256) + ")' />"
  data += "</svg>";

  fileName = path.join(__dirname, "../arcaneImages/", fileName);
  fs.writeFile(fileName, data, (err) => {
    callback(err);
  });
}
