const express = require('express');
const passport = require('passport');
const jwt = require('jsonwebtoken');
const async = require('async');

const tools = require('../tools/tools');
const routeError = tools.routeError;
const User = require('../models/user');
const Workspace = require('../models/workspace');

const router = express.Router();

module.exports = router;


router.get('/', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  User.getUserWorkspaces(req.user.id, (err, workspaces) => {
    routeError(err, res, () => {
      User.getUserElements(req.user.id, (err, elements) => {
        routeError(err, res, () => {
          User.getUserAura(req.user.id, (err, aura) => {
            routeError(err, res, () => {
              if(workspaces.length <= 0) {
                res.json({empty: true, elementList: elements, aura: aura});
              }
              else {
                Workspace.getWorkspace(workspaces[0].id, (err, w) => {
                  routeError(err, res, () => {
                    //compute element quantity
                    let used = tools.getElementsOf(w.content);
                    tools.computeElemQuantity(elements, used, -1);
                    res.json({list: workspaces, current: w, elementList: elements, aura: aura});
                  });
                });
              }
            });
          });
        });
      });
    });
  });
});

router.post('/newWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Workspace.createWorkspace(req.body, req.user.username, null, (err, workspace) => {
    routeError(err, res, () => {
      let w = {id: workspace.id, name: workspace.name};
      User.addWorkspace(req.user.id, w, (err) => {
        routeError(err, res, () => {
          User.getUserElements(req.user.id, (err, elements) => {
            routeError(err, res, () => {
              res.json({workspace: workspace, elements: elements});
            });
          });
        });
      });
    });
  });
});

router.post('/changeWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Workspace.getWorkspace(req.body.id, (err, workspace) => {
    routeError(err, res, () => {
      User.getUserElements(req.user.id, (err, elements) => {
        routeError(err, res, () => {
          let used = tools.getElementsOf(workspace.content);
          tools.computeElemQuantity(elements, used, -1);
          res.json({workspace: workspace, elements: elements});
        });
      });
    });
  });
});

router.post('/deleteWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Workspace.deleteWorkspace(req.body.id, (err) => {
    routeError(err, res, () => {
      User.updateUser('workspaces', JSON.stringify(req.body.list), req.user.id, (err) => {
        routeError(err, res, () => {
          if(req.body.list.length == 0) {
            res.json({empty: true});
          }
          else {
            Workspace.getWorkspace(req.body.list[0].id, (err, workspace) => {
              routeError(err, res, () => {
                User.getUserElements(req.user.id, (err, elements) => {
                  routeError(err, res, () => {
                    let used = tools.getElementsOf(workspace.content);
                    tools.computeElemQuantity(elements, used, -1);
                    res.json({workspace: workspace, elements: elements});
                  });
                });
              });
            });
          }
        });
      });
    });
  });
});

router.post('/renameWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Workspace.updateWorkspace('name', req.body.name, req.body.id, (err) => {
    routeError(err, res, () => {
      User.updateUser('workspaces', JSON.stringify(req.body.list), req.user.id, (err) => {
        routeError(err, res, () => {
          res.json({success: true});
        });
      });
    });
  });
});

router.post('/moveWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  User.updateUser('workspaces', JSON.stringify(req.body), req.user.id, (err) => {
    routeError(err, res, () => {
      res.json({success: true});
    });
  });
});

router.post('/saveWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Workspace.updateWorkspace('content', JSON.stringify(req.body.content), req.body.id, (err) => {
    routeError(err, res, () => {
      User.getUserElements(req.user.id, (err, elements) => {
        routeError(err, res, () => {
          let used = tools.getElementsOf(req.body.content);
          tools.computeElemQuantity(elements, used, -1);
          res.json({success: true, elem: elements});
        });
      });
    });
  });
});

router.post('/duplicateWorkspace', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Workspace.duplicateWorkspace(req.body.newName, req.body.id, (err, w) => {
    routeError(err, res, () => {
      User.addWorkspace(req.user.id, {name: w.name, id: w.id}, (err) => {
        routeError(err, res, () => {
          res.json({name: w.name, id: w.id});
        });
      });
    });
  });
});
