const express = require('express');
const passport = require('passport');
const jwt = require('jsonwebtoken');
const async = require('async');
const sem = require('semaphore')(1);

const routeError = require('../tools/tools').routeError;
const tools = require('../tools/tools');
const User = require('../models/user');
const Arcane = require('../models/arcane');
const Page = require('../models/page');
const Workspace = require('../models/workspace');
const mem = require('../models/mem');

const router = express.Router();

module.exports = router;


router.get('/', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  User.getUserPages(req.user.id, (err, pages) => {
    routeError(err, res, () => {
      Page.getPage(pages[0].id, (err, p) => {
        routeError(err, res, () => {
          getArcanesOfPage(p, (err, pRes) => {
            routeError(err, res, () => {
              Page.recommendation(p.id, req.user, (err, r) => {
                routeError(err, res, () => {
                  User.getUserAura(req.user.id, (err, aura) => {
                    routeError(err, res, () => {
                      res.json({list: pages, recommendation: r, current: pRes, aura: aura});
                    });
                  });
                });
              });
            });
          });
        });
      });
    });
  });
});


router.post('/newPage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Page.createPage(req.body, req.user.username, 0, (err, page) => {
    routeError(err, res, () => {
      let p = {id: page.id, name: page.name};
      User.addPage(req.user.id, p, (err) => {
        routeError(err, res, () => {
          res.json(p);
        });
      });
    });
  });
});

router.post('/changePage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Page.getPage(req.body.id, (err, page) => {
    routeError(err, res, () => {
      getArcanesOfPage(page, (err, p) => {
        routeError(err, res, () => {
          Page.recommendation(p.id, req.user, (err, r) => {
            routeError(err, res, () => {
              res.json({recommendation: r, current: p});
            });
          });
        });
      });
    });
  });
});

router.post('/deletePage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  sem.take(() => {
    Page.removeSimilarity(req.body.id, (err) => {
      routeError(err, res, () => {
        mem.deletePage(req.body.id, (err) => {
          routeError(err, res, () => {
            Page.deletePage(req.body.id, (err) => {
              routeError(err, res, () => {
                User.updateUser('pages', JSON.stringify(req.body.list), req.user.id, (err) => {
                  routeError(err, res, () => {
                    Page.getPage(req.body.list[0].id, (err, page) => {
                      routeError(err, res, () => {
                        getArcanesOfPage(page, (err, p) => {
                          routeError(err, res, () => {
                            Page.recommendation(p.id, req.user, (err, r) => {
                              routeError(err, res, () => {
                                sem.leave();
                                res.json({page: p, recommendation: r});
                              });
                            });
                          });
                        });
                      });
                    });
                  });
                });
              });
            });
          });
        });
      });
    });
  });
});

router.post('/renamePage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Page.updatePage('name', req.body.name, req.body.id, (err) => {
    routeError(err, res, () => {
      User.updateUser('pages', JSON.stringify(req.body.list), req.user.id, (err) => {
        routeError(err, res, () => {
          res.json({success: true});
        });
      });
    });
  });
});

router.post('/movePage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  User.updateUser('pages', JSON.stringify(req.body), req.user.id, (err) => {
    routeError(err, res, () => {
      res.json({success: true});
    });
  });
});

router.post('/deleteFromPage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  sem.take(() => {
    Page.removeArcane(req.body.pageId, req.body.arcaneId, (err) => {
      routeError(err, res, () => {
        let value = 0;
        if(req.body.recommendation) {
          value = -1;
        }
        Page.updateSimilarity(value, req.body.pageId, req.body.arcaneId, (err) => {
          routeError(err, res, () => {
            if(req.body.recommendation) {
              Page.recommendation(req.body.pageId, req.user, (err, r) => {
                routeError(err, res, () => {
                  sem.leave();
                  res.json({recommendation: r});
                });
              });
            }
            else {
              sem.leave();
              res.json({success: true});
            }
          });
        });
      });
    });
  });
});

router.post('/addToPage', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  sem.take(() => {
    async.forEach(req.body.pages, (p, callback) => {
      Page.addArcane(p.id, req.body.arcaneId, (err) => {
        routeError(err, res, () => {
          Page.updateSimilarity(1, p.id, req.body.arcaneId, (err) => {
            routeError(err, res, () => {
              if(req.body.recommendation) {
                tools.addArcaneToPage("Arcanes connus", req.user, req.body.arcaneId, (err) => {
                  routeError(err, res, () => {
                    callback();
                  });
                });
              }
              else {
                callback();
              }
            });
          });
        });
      });
    }, () => {
      if(req.body.recommendation) {
        Page.recommendation(req.body.pageId, req.user, (err, r) => {
          routeError(err, res, () => {
            tools.receiveAuraForRecommendation(req.body.arcaneId, req.user, (err, aura) => {
              routeError(err, res, () => {
                sem.leave();
                res.json({recommendation: r, aura: aura});
              });
            });
          });
        });
      }
      else {
        sem.leave();
        res.json({success: true});
      }
    });
  });
});

router.post('/pagePossibilities', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  Page.getAllPageFrom(req.user.username, (err, pages) => {
    routeError(err, res, () => {
      let arr = new Array();
      for(let p of pages) {
        if(!p.isStatic && p.content.indexOf(req.body.arcaneId) == -1) {
          arr.push({name: p.name, id: p.id});
        }
      }
      res.json(arr);
    });
  });
});

router.post('/decrypt', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  User.getUserAura(req.user.id, (err, aura) => {
    aura -= req.body.arcane.cost;
    if(aura >= 0) {
      User.getUserElements(req.user.id, (err, elem) => {
        routeError(err, res, () => {
          let used = tools.getElementsOf(req.body.arcane.schema);
          tools.computeElemQuantity(elem, used, 1);
          User.updateUser('elements', JSON.stringify(elem), req.user.id, (err) => {
            routeError(err, res, () => {
              //add a workspace
              Workspace.createWorkspace('W' + String(Date.now()), req.user.username, req.body.arcane.schema, (err, workspace) => {
                routeError(err, res, () => {
                  let w = {id: workspace.id, name: workspace.name};
                  User.addWorkspace(req.user.id, w, (err) => {
                    routeError(err, res, () => {
                      User.updateAura(req.user.id, -req.body.arcane.cost, (err, newAura) => {
                        routeError(err, res, () => {
                          res.json({success: true, aura: newAura});
                        });
                      });
                    });
                  });
                });
              });
            });
          });
        });
      });
    }
    else {
      res.json({success: false, msg: "Vous ne possédez pas assez d'aura pour décrypter cet arcane."});
    }
  });
});

function getArcanesOfPage(p, callback) {
  if(p.content.length > 0) {
    Arcane.getArcanes('id', p.content, (err, arcanes) => {
      if(err) {
        callback(err, undefined);
      }
      else {
        p.content = arcanes;
        callback(null, p);
      }
    });
  }
  else {
    callback(null, p);
  }
}
