const express = require('express');
const passport = require('passport');
const jwt = require('jsonwebtoken');
const bcrypt = require('bcryptjs');

const routeError = require('../tools/tools').routeError;
const configdb = require('../config/database');
const User = require('../models/user');
const Arcane = require('../models/arcane');
const Page = require('../models/page');

const router = express.Router();


//register POST
router.post('/register', (req, res, next) => {
  let newUser = new User(req.body.username, req.body.email, req.body.password);
  User.addUser(newUser, (err, result) => {
    routeError(err, res, () => {
      res.json(result);
    });
  });
});

//authenticate POST
router.post('/authenticate', (req, res, next) => {
  const username = req.body.username;
  const password = req.body.password;
  User.getUser('username', username, (err, user) => {
    routeError(err, res, () => {
      if(!user){
        res.json({
          success: false,
          msg: "Nom d'utilisateur ou mot de passe érroné"
        });
      }
      else {
        //if user found check password
        //if password match create a token for pages with authentification requirement
        User.comparePassword(password, user.password, (err, isMatch) => {
          routeError(err, res, () => {
            if(isMatch) {
              const token = jwt.sign(JSON.parse(JSON.stringify(user)), configdb.secret, {expiresIn: 28800});
              res.json({
                success: true,
                token: 'Bearer ' + token,
                user: {
                  username: user.username,
                  email: user.email
                }
              });
            }
            else {
              res.json({
                success: false,
                msg: "Nom d'utilisateur ou mot de passe érroné"
              });
            }
          });
        });
      }
    });
  });
});


//profile GET
router.get('/profile', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  let user = {
    username: req.user.username,
    email: req.user.email,
    totalArcane: 0,
    createArcane: 0,
    classArcane: 0
  };
  
  Page.getPageByName("Arcanes connus", req.user.username, (err, known) => {
    routeError(err, res, () => {
      if(known.content.length <= 0) {
        res.json({user});
      }
      else {
        user.totalArcane = known.content.length;
        Arcane.getArcanes('id', known.content, (err, arcanes) => {
          routeError(err, res, () => {
            for(let a of arcanes) {
              if(a.author == req.user.username) {
                user.createArcane ++;
              }
            }
            Page.getAllPageFrom(req.user.username, (err, pages) => {
              routeError(err, res, () => {
                let seen = [];
                // get the number of classified arcanes
                for(let p of pages) {
                  if(!p.isStatic) {
                    for(let id of p.content) {
                      for(let a of arcanes) {
                        if(id == a.id && !seen.includes(id)) {
                          seen.push(id);
                          user.classArcane ++;
                        }
                      }
                    }
                  }
                }
                res.json({user});
              });
            });
          });
        });
      }
    });
  });
});

router.post('/updateUserInfo', passport.authenticate('jwt', {session:false}), (req, res, next) => {
  User.getUser('id', req.user.id, (err, user) => {
    routeError(err, res, () => {
      if(!user){
        res.json({
          success: false,
          msg: "Impossible d'éffectuer cette action pour le moment"
        });
      }
      else {
        User.comparePassword(req.body.password, user.password, (err, isMatch) => {
          routeError(err, res, () => {
            // password match
            if(isMatch) {
              //e-mail change request
              if(req.body.type == 'email') {
                //check if this e-mail isn't already used
                User.getUser('email', req.body.email, (err, dupUser) => {
                  routeError(err, res, () => {
                    if(dupUser) {
                      res.json({success: false, msg: "Adresse e-mail déjà utilisée"});
                    }
                    else {
                      User.updateUser('email', req.body.email, req.user.id, (err) => {
                        routeError(err, res, () => {
                          req.user.email = req.body.email;
                          res.json({success: true});
                        });
                      });
                    }
                  });
                });
              }
              //password change request
              else if(req.body.type == 'password') {
                bcrypt.genSalt(10, (err, salt) => {
                  routeError(err, res, () => {
                    //crypting password
                    bcrypt.hash(req.body.newPassword, salt, (err, hash) => {
                      routeError(err, res, () => {
                        //push new password
                        User.updateUser('password', hash, req.user.id, (err) => {
                          routeError(err, res, () => {
                            req.user.password = "";
                            res.json({success: true});
                          });
                        });
                      });
                    });
                  });
                });
              }
              else {
                routeError("ERROR : incorrect information update type", res, () => {});
              }
            }
            else {
              res.json({success: false, msg: "Mot de passe érroné"});
            }
          });
        });
      }
    });
  });
});

module.exports = router;
