const express = require('express');
const path = require('path');
const bodyParser = require('body-parser');
const cors = require('cors');
const passport = require('passport');
const osc = require('osc');
const pubsub = require('pubsub-js');

const https = require('https');
const helmet = require('helmet');

const udp = require('./supercollider/udp');
const users = require('./routes/users');
const generate = require('./routes/generate');
const laboratory = require('./routes/laboratory');
const spellbook = require('./routes/spellbook');
const db = require('./db');

const app = express();

//middlewares
app.use(helmet());
app.use(cors());
app.use(bodyParser.json());
app.use(bodyParser.text());
//static files
app.use(express.static(path.join(__dirname, 'public')));
app.use('/sounds', express.static(__dirname + '/sounds'));
app.use('/arcanes', express.static(__dirname + '/arcaneImages'));

app.use(passport.initialize());
app.use(passport.session());
require('./config/passport')(passport);

//routes
app.use('/users', users);
app.use('/generate', generate);
app.use('/laboratory', laboratory);
app.use('/spellbook', spellbook);

let openApp = null;

module.exports.startApp = function(port, callback) {
  udp.open();

  udp.on('ready', () => {
    console.log("OSC service ready");
    //database connection
    db.connect((err) => {
      if(err) {
        callback(err);
      }
      else {
        console.log("Connected to database");
        //HTTPS
        /*const credentials = {key: fs.readFileSync('key.pem'), cert: fs.readFileSync('cert.pem')};
        openApp = https.createServer(credentials, app).listen(port, () => {
          callback(null);
        });*/

        //start listening
        openApp = app.listen(port, () => {
          callback(null);
        });
      }
    });
  });
}

module.exports.closeApp = function() {
  openApp.close();
}

module.exports.app = app;

//############### SUPERCOLLIDER ##########################

udp.on("error", function (err) {
  console.log("OSC error : " + err);
});

//for sc response
udp.on('message', (msg) => {
  const data = JSON.parse(JSON.stringify(msg));
  pubsub.publish(data.args[0].value, data.args[1].value);
});
