const assert = require('chai').assert;
const Arcane = require('../../models/arcane');

module.exports = describe('arcane', () => {
  let id = -1;

  it('Should add a new arcane', (done) => {
    const newArcane = new Arcane('author', 'fileName', 'formula', {schema: 'schema'}, 100);
    Arcane.createArcane(newArcane, (err, arcaneId) => {
      assert.isNull(err, err);
      assert.isDefined(arcaneId, 'Result id not defined');
      assert.isAbove(arcaneId, 0, 'Wrong id value : ' + arcaneId);
      id = arcaneId;
      done();
    });
  });

  it('Should get an arcane', (done) => {
    Arcane.getArcane('id', id, (err, arcane) => {
      assert.isNull(err, err);
      assert.isDefined(arcane, 'Arcane undefined');
      assert.equal(arcane.id, id, 'Wrong arcane returned');
      done();
    });
  });

  it('Should get an arcane list', (done) => {
    const newArcane = new Arcane('author2', 'fileName2', 'formula2', {schema: 'schema2'}, 100);
    Arcane.createArcane(newArcane, (err, arcaneId) => {
      assert.isNull(err, err);
      assert.isDefined(arcaneId, 'Result id not defined');
      assert.isAbove(arcaneId, 0, 'Wrong id value : ' + arcaneId);
      let idList = new Array(id, arcaneId);

      Arcane.getArcanes('id', idList, (err, arcanes) => {
        assert.isNull(err, err);
        assert.lengthOf(arcanes, idList.length, 'Did not get all the arcanes');
        done();
      });
    });
  });

  it('Should get all the arcanes except an exclusion list', (done) => {
    Arcane.getAllExceptList(id, (err, arcanes) => {
      assert.isNull(err, err);
      assert.isDefined(arcanes);

      Arcane.getAllExceptList([], (err, arcanes) => {
        assert.isNull(err, err);
        assert.isDefined(arcanes);
        assert.lengthOf(arcanes, 2);
        done();
      });
    });
  });
});
