const assert = require('chai').assert;
const mem = require('../../models/mem');

module.exports = describe('mem', () => {

  it('Should create new membership cases', (done) => {
    mem.createMem(1, 1, 1, (err) => {
      assert.isNull(err, err);
      mem.createMem(2, 1, -1, (err) => {
        assert.isNull(err, err);
        mem.createMem(1, 2, 1, (err) => {
          assert.isNull(err, err);
          done();
        });
      });
    });
  });

  it('Should get the created memberships', (done) => {
    mem.getMem(1, 1, (err, val) => {
      assert.isNull(err, err);
      assert.equal(val, 1);
      mem.getMem(2, 1, (err, val) => {
        assert.isNull(err, err);
        assert.equal(val, -1);
        mem.getMem(1, 2, (err, val) => {
          assert.isNull(err, err);
          assert.equal(val, 1);
          done();
        });
      });
    });
  });

  it('Should return 0 if a membership doesn\'t exist', (done) => {
    mem.getMem(123, 123, (err, val) => {
      assert.isNull(err, err);
      assert.equal(val, 0);
      done();
    });
  });

  it('Should get the arcanes ids of a page', (done) => {
    mem.getArcanesOfPage(1, (err, arcanes) => {
      assert.isNull(err, err);
      assert.lengthOf(arcanes, 2);
      done();
    });
  });

  it('Should get an object with all pages containing an arcane', (done) => {
    mem.getPagesWithArcane(1, (err, p) => {
      assert.isNull(err, err);
      assert.equal(p[1], 1);
      assert.equal(p[2], 1);
      done();
    });
  });

  it('Should get all membership cases within a list of arcanes and pages', (done) => {
    mem.getCommendableMems([1, 2], [2], (err, mems) => {
      assert.isNull(err, err);
      assert.lengthOf(mems, 1);
      assert.equal(mems[0].arcane, 1);
      assert.equal(mems[0].page, 2);
      done();
    });
  });

  it('Should update a membership case value', (done) => {
    mem.updateValue(2, 1, 1, (err) => {
      assert.isNull(err, err);
      mem.getMem(2, 1, (err, val) => {
        assert.isNull(err, err);
        assert.equal(val, 1);
        done();
      });
    });
  });

  it('Should delete a membership case', (done) => {
    mem.deleteMem(1, 2, (err) => {
      assert.isNull(err, err);
      mem.getMem(1, 2, (err, val) => {
        assert.isNull(err, err);
        assert.equal(val, 0);
        done();
      });
    });
  });

  it('Should, delete all membership cases for a page', (done) => {
    mem.deletePage(1, (err) => {
      assert.isNull(err, err);
      mem.getMem(1, 1, (err, val) => {
        assert.isNull(err, err);
        assert.equal(val, 0);
        mem.getMem(2, 1, (err, val) => {
          assert.isNull(err, err);
          assert.equal(val, 0);
          done();
        });
      });
    });
  });

});
