const assert = require('chai').assert;
const Page = require('../../models/page');

module.exports = describe('page', () => {

  let p = null;

  it('Should create a new page', (done) => {
    Page.createPage('testPage', 'me', 0, (err, page) => {
      assert.isNull(err, err);
      assert.equal(page.name, 'testPage');
      assert.notEqual(page.id, -1);
      assert.equal(page.author, 'me');
      assert.equal(page.isStatic, 0);
      assert.isEmpty(page.content);
      p = page;
      done();
    });
  });

  it('Should get the page with id', (done) => {
    Page.getPage(p.id, (err, page) => {
      assert.isNull(err, err);
      assert.equal(page.name, 'testPage');
      assert.notEqual(page.id, -1);
      assert.equal(page.author, 'me');
      assert.equal(page.isStatic, 0);
      assert.isEmpty(page.content);
      done();
    });
  });

  it('Should get the page with name and author', (done) => {
    Page.getPageByName(p.name, p.author, (err, page) => {
      assert.isNull(err, err);
      assert.equal(page.name, 'testPage');
      assert.notEqual(page.id, -1);
      assert.equal(page.author, 'me');
      assert.equal(page.isStatic, 0);
      assert.isEmpty(page.content);
      done();
    });
  });

  it('Should get all the pages of an author', (done) => {
    Page.createPage('page2', 'me', 0, (err, page) => {
      assert.isNull(err, err);
      Page.getAllPageFrom(p.author, (err, pages) => {
        assert.isNull(err, err);
        assert.lengthOf(pages, 2);
        done();
      });
    });
  });

  it('Should add an arcane to the page', (done) => {
    Page.addArcane(p.id, 1000, (err) => {
      assert.isNull(err, err);
      done();
    });
  });

  it('Should test if an arcane is in the page', (done) => {
    Page.isInPage(p.id, 1000, (err, isIn) => {
      assert.isNull(err, err);
      assert.isTrue(isIn);
      Page.isInPage(p.id, 123, (err, isIn) => {
        assert.isNull(err, err);
        assert.isFalse(isIn);
        done();
      });
    });
  });

  it('Should update page name', (done) => {
    Page.updatePage('name', 'newPageName', p.id, (err) => {
      assert.isNull(err, err);
      Page.getPage(p.id, (err, page) => {
        assert.isNull(err, err);
        assert.equal(page.name, 'newPageName');
        assert.equal(p.id, page.id);
        assert.equal(p.author, page.author);
        done();
      });
    });
  });

  it('Should remove the arcane from the page', (done) => {
    Page.removeArcane(p.id, 1000, (err) => {
      assert.isNull(err, err);
      Page.isInPage(p.id, 1000, (err, isIn) => {
        assert.isNull(err, err);
        assert.isFalse(isIn);
        done();
      });
    });
  });

  it('Should delete a page', (done) => {
    Page.deletePage(p.id, (err) => {
      assert.isNull(err, err);
      Page.getAllPageFrom(p.author, (err, pages) => {
        assert.isNull(err, err);
        assert.lengthOf(pages, 1);
        done();
      });
    });
  });


});
