const assert = require('chai').assert;
const expect = require('chai').expect;
const User = require('../../models/user');


module.exports = describe('user', () => {
  let testUser = undefined;

  it('Should create a new user', (done) => {
    let newUser = new User('nameTest', 'mailTest@test.com', 'pwdTest123');
    User.addUser(newUser, (err, result) => {
      assert.isNull(err, err);
      expect(result.success, result.msg).to.be.true;
      done();
    });
  });

  it('Should not add the same user', (done) => {
    let newUser = new User('nameTest', 'mailTest@test.com', 'pwdTest123');
    User.addUser(newUser, (err, result) => {
      assert.isNull(err, err);
      expect(result.success).to.be.false;
      done();
    });
  });

  it('Should get user information', (done) => {
    User.getUser('username', 'nameTest', (err, user) => {
      testUser = user;
      assert.isNull(err, err);
      assert.isDefined(user, 'user undefined');
      assert.notEqual(testUser.id, -1, 'wrong user id');
      assert.equal(testUser.username, 'nameTest', 'wrong username');
      assert.equal(testUser.email, 'mailTest@test.com', 'wrong user e-mail');
      done();
    });
  });

  it('Candidate password should match database hashed password', (done) => {
    User.comparePassword('pwdTest123', testUser.password, (err, isMatch) => {
      assert.isNull(err, err);
      assert.isTrue(isMatch, 'passwords not matching');
      done();
    });
  });

  it('Should add a new workspace to this user workspaces list', (done) => {
    User.addWorkspace(testUser.id, {name: 'w1', id: 1000}, (err) => {
      assert.isNull(err, err);
      done();
    });
  });

  it('Should get user workspaces', (done) => {
    User.getUserWorkspaces(testUser.id, (err, workspaces) => {
      assert.isNull(err, err);
      assert.isDefined(workspaces, 'workspaces undefined');
      assert.equal(workspaces[0].id, 1000);
      done();
    });
  });

  it('Should add a new page to this user pages list', (done) => {
    User.addWorkspace(testUser.id, {name: 'page1', id: 1000}, (err) => {
      assert.isNull(err, err);
      done();
    });
  });

  it('Should get user spell book pages', (done) => {
    User.getUserPages(testUser.id, (err, pages) => {
      assert.isNull(err, err);
      assert.isDefined(pages, 'pages undefined');
      done();
    });
  });

  it('Should get user elements', (done) => {
    User.getUserElements(testUser.id, (err, elements) => {
      assert.isNull(err, err);
      assert.isDefined(elements, 'Elements undefined');
      done();
    });
  });

  it('Should update user fields', (done) => {
    User.updateUser('username', 'testName', testUser.id, (err) => {
      assert.isNull(err, err);
      User.getUser('username', 'testName', (err, user) => {
        assert.isNull(err, err);
        assert.isDefined(user, 'user undefined');
        assert.equal(user.username, 'testName');
        User.getUser('username', 'nameTest', (err, user) =>  {
          assert.isNull(err, err);
          assert.isUndefined(user, 'old username should not be found');
          done();
        });
      });
    });
  });

});
