const assert = require('chai').assert;
const Workspace = require('../../models/workspace');

module.exports = describe('workspace', () => {

  let w = null;

  it('Should create a new workspace', (done) => {
    Workspace.createWorkspace('testWorkspace', 'me', (err, workspace) => {
      assert.isNull(err, err);
      assert.equal(workspace.name, 'testWorkspace');
      assert.equal(workspace.author, 'me');
      assert.notEqual(workspace.id, -1);
      assert.isDefined(workspace.content);
      w = workspace;
      done();
    });
  });

  it('Should get the workspace with the id', (done) => {
    Workspace.getWorkspace(w.id, (err, workspace) => {
      assert.isNull(err, err);
      assert.equal(workspace.name, w.name);
      assert.equal(workspace.author, w.author);
      assert.equal(workspace.id, w.id);
      assert.isDefined(workspace.content);
      done();
    });
  });

  it('Should update the workspace name', (done) => {
    Workspace.updateWorkspace('name', 'newWorkspaceName', w.id, (err) => {
      assert.isNull(err, err);
      Workspace.getWorkspace(w.id, (err, workspace) => {
        assert.isNull(err, err);
        assert.equal(workspace.name, 'newWorkspaceName');
        assert.equal(workspace.author, w.author);
        assert.equal(workspace.id, w.id);
        w = workspace;
        done();
      });
    });
  });

  it('Should duplicate the workspace', (done) => {
    Workspace.duplicateWorkspace('dup', w.id, (err, workspace) => {
      assert.isNull(err, err);
      assert.notEqual(workspace.name, w.name);
      assert.equal(workspace.name, 'dup');
      assert.equal(workspace.author, w.author);
      assert.notEqual(workspace.id, w.id);
      assert.equal(JSON.stringify(workspace.content), JSON.stringify(w.content));
      done();
    });
  });

  it('Should delete the original workspace', (done) => {
    Workspace.deleteWorkspace(w.id, (err) => {
      assert.isNull(err, err);
      Workspace.getWorkspace(w.id, (err, workspace) => {
        assert.isDefined(err);
        assert.equal(err, "ERROR: no workspace found");
        assert.isUndefined(workspace);
        done();
      });
    });
  });
});
