const chai = require('chai');
const chaiHttp = require('chai-http');
const app = require('../../server').app;
const udp = require('../../supercollider/udp');

chai.use(chaiHttp);

module.exports = describe('generate routes', () => {

  let formula = {
    size: [{type: "C", value: 8}],
    time: [{type: "T", value: 0.1}, {type: "T", value: 1.9}],
    amp: [{type: "env", value: [{type: "K", value: -1}, {type: "K/nil", value: "nil"}, {type: "K", value: 1}]}, {type: "mod", value: null}],
    freq: [{type: "param", value: [{type: "K/env", value: 0.2}, {type: "mod", value: null}]}],
    mass: [{type: "fonc1", value: [{type: "F1", value: "brt"}, {type: "param", value: [{type: "K/env", value: 1}, {type: "mod", value: null}]}]}],
    ton: [{type: "fonc2", value: null}],
    color: [{type: "param", value:null}, {type: "param", value:null}, {type: "param", value:null}]
  }

  let authToken = null;

  describe('POST generate', () => {

    it('Should create a new user', (done) => {
      chai.request(app)
      .post('/users/register')
      .set('content-type', 'application/json')
      .send({
          username: 'testGen',
          email: 'testGen@mail.com',
          password: 'UserPwd123'
      })
      .end((err, res) => {
        chai.expect(res).to.have.status(200);
        chai.assert.isNull(err, err);
        chai.request(app)
        .post('/users/authenticate')
        .set('content-type', 'application/json')
        .send({
            username: 'testGen',
            password: 'UserPwd123'
        })
        .end((err, res) => {
          chai.expect(res).to.have.status(200);
          chai.assert.isNull(err, err);
          authToken = res.body.token;
          done();
        });
      });
    });

    it('Should create a new arcane', (done) => {
      udp.open();

      udp.on('ready', () => {
        chai.request(app)
        .post('/generate')
        .set('Authorization', authToken)
        .set('Content-Type', 'application/json')
        .set('Cache-control', 'no-cache')
        .send({
          content: formula
        })
        .end((err, res) => {
          chai.assert.isNull(err, err);
          chai.assert.isTrue(res.body.success);
          done();
        });
      });
    });

    it('Should not create an arcane if this formula already exist', (done) => {
      chai.request(app)
      .post('/generate')
      .set('Authorization', authToken)
      .set('Content-Type', 'application/json')
      .set('Cache-control', 'no-cache')
      .send({
        content: formula
      })
      .end((err, res) => {
        chai.assert.isNull(err, err);
        chai.assert.isFalse(res.body.success);
        done();
      });
    });
  });
});
