const express = require('express');
const Page = require('../models/page');
const User = require('../models/user');
const Arcane = require('../models/arcane');

module.exports.routeError = function(err, res, callback) {
  if(err) {
    res.json({error: err});
    console.log(err);
  }
  else {
    callback();
  }
}

module.exports.randi = function(max) {
  return Math.floor(Math.random() * Math.floor(max));
}

module.exports.addArcaneToPage = function(pageName, user, arcaneId, callback) {
  Page.getPageByName(pageName, user.username, (err, page) => {
    if(err) {
      callback(err);
    }
    else {
      Page.addArcane(page.id, arcaneId, (err) => {
        callback(err);
      });
    }
  });
}

module.exports.getElementsOf = function(content) {
  let arr = new Array();
  for(let c in content) {
    switch(c) {
      case "size":
        if(content.size[0].value != null) {
          arr.push(content.size[0]);
        }
      break;
      case "time":
        for(let e of content[c]) {
          if(e.value != null) {
            arr.push(e);
          }
        }
      break;
      default:
        recGetElements(content[c], arr);
      break;
    }
  }
  return arr;
}

module.exports.receiveAuraForRecommendation = function(arcaneId, currentUser, callback) {
  Arcane.getArcane('id', arcaneId, (err, arcane) => {
    if(err) {
      callback(err, 0);
    }
    else {
      User.getUser('username', arcane.author, (err, user) => {
        if(err) {
          callback(err, 0);
        }
        else {
          User.getUserAura(user.id, (err, userAura) => {
            if(err) {
              callback(err, 0);
            }
            else {
              let addAura = Math.floor((User.MAX_AURA - userAura) * 0.03);
              User.updateAura(user.id, addAura, (err, newAura) => {
                if(err) {
                  callback(err, 0);
                }
                else {
                  User.getUserAura(currentUser.id, (err, currentUserAura) => {
                    if(err) {
                      callback(err, 0);
                    }
                    else {
                      addAura = Math.floor((User.MAX_AURA - currentUserAura) * 0.02);
                      User.updateAura(currentUser.id, addAura, (err, newAura) => {
                        callback(err, newAura);
                      });
                    }
                  });
                }
              });
            }
          });
        }
      });
    }
  });

}

//q = -1 or +1
module.exports.computeElemQuantity = function(userElem, used, q) {
  for(let u of used) {
    for(let e of userElem) {
      if(e.name == u.type) {
        if(e.content != null) {
          for(let l of e.content) {
            if(l.name == u.value || l.name == u.name) {
              l.quantity += q;
            }
          }
        }
        else {
          e.quantity += q;
        }
      }
    }
  }
}

module.exports.elementsCost = function(elem) {
  let T = C = K = F = 0;
  for(let e of elem) {
    switch(e.type) {
      case "T":
        if(e.value < 0.1) {
          T += 5;
        }
        else if(e.value >= 0.1 && e.value < 1) {
          T += 7;
        }
        else {
          T += 10;
        }
      break;
      case "C":
        C = e.value / 4;
      break;
      case "K":
        K += 1;
      break;
      case "mod":
      case "fonc1":
      case "fonc2":
      case "color":
        F += 5;
      break;
      default:
      break;
    }
  }
  return T*C*(K+F);
}

function recGetElements(content, arr) {
  for(let c of content) {
    switch (c.type) {
      case "K":
      case "K/nil":
        if(c.value != null) {
          arr.push({type: "K", value: c.value});
        }
      break;
      case "env":
        if(c.value != null) {
          recGetElements(c.value, arr);
        }
      break;
      case "K/env":
        if(c.value != null) {
          if(Array.isArray(c.value)) {
            arr.push({type: "env", value: null});
            recGetElements(c.value, arr);
          }
          else {
            arr.push({type: "K", value: c.value});
          }
        }
      break;
      case "param":
        if(c.value != null) {
            recGetElements(c.value, arr);
        }
      break;
      case "mod":
      case "color":
      case "fonc1":
      case "fonc2":
      case "col":
        if(c.value != null) {
          arr.push({type: c.type, value: c.value[0].value});
          recGetElements(c.value, arr);
        }
      break;
      default:
      break;
    }
  }
}
